@@ -30,15 +30,19 @@
 // KDE includes
 #include <kstandarddirs.h>
 #include <kdebug.h>
+#include <kapplication.h>
 
 // application specific includes
 #include "smb4kbookmarkhandler.h"
 #include "smb4kdefs.h"
+#include "smb4kerror.h"
 #include "smb4kglobal.h"
 using namespace Smb4KGlobal;
 
 
-Smb4KBookmarkHandler::Smb4KBookmarkHandler( QObject *parent, const char *name ) : QObject( parent, name )
+Smb4KBookmarkHandler::Smb4KBookmarkHandler( QValueList<Smb4KHostItem *> *hosts,
+QObject *parent, const char *name )
+: QObject( parent, name ), m_hosts( hosts )
 {
   // First we need the directory.
   KStandardDirs *stddir = new KStandardDirs();
@@ -63,6 +67,9 @@ Smb4KBookmarkHandler::~Smb4KBookmarkHandler()
   }
 
   m_bookmarks.clear();
+
+  // Do not delete m_hosts here, because it is either NULL
+  // or it is handled outside of this class.
 }
 
 
@@ -74,13 +81,13 @@ void Smb4KBookmarkHandler::addBookmark( Smb4KBookmark *bookmark )
 {
   if ( QString::compare( bookmark->type(), "Printer" ) == 0 )
   {
-    showCoreError( ERROR_WRONG_BOOKMARK_TYPE );
+    Smb4KError::error( ERROR_BOOKMARK_PRINTER );
     return;
   }
 
   if ( QString::compare( bookmark->share(), "homes" ) == 0 )
   {
-    QString share = specifyUser( bookmark->host() );
+    QString share = specifyUser( bookmark->host(), kapp->mainWidget() ? kapp->mainWidget() : 0, "SpecifyUser" );
 
     bookmark->setShareName( share );
   }
@@ -145,7 +152,7 @@ void Smb4KBookmarkHandler::writeBookmarkList( const QValueList<Smb4KBookmark *>
   }
   else
   {
-    showCoreError( ERROR_WRITING_FILE, QDir::currentDirPath()+"/"+file.name() );
+    Smb4KError::error( ERROR_WRITING_FILE, QDir::currentDirPath()+"/"+file.name() );
     return;
   }
 
@@ -201,6 +208,17 @@ void Smb4KBookmarkHandler::loadBookmarks()
 
     emit bookmarksUpdated();
   }
+  else
+  {
+    if ( file.exists() )
+    {
+      Smb4KError::error( ERROR_READING_FILE, file.name() );
+    }
+    else
+    {
+      // Do nothing if the file does not exist.
+    }
+  }
 }
 
 
@@ -208,8 +226,12 @@ void Smb4KBookmarkHandler::loadBookmarks()
    Return a bookmark object from the list
 ****************************************************************************/
 
-Smb4KBookmark *Smb4KBookmarkHandler::getBookmark( const QString &bookmark )
+Smb4KBookmark *Smb4KBookmarkHandler::findBookmark( const QString &bookmark )
 {
+  // Update the bookmarks:
+  update();
+
+  // Find the bookmark:
   QValueListConstIterator<Smb4KBookmark *> it;
 
   for ( it = m_bookmarks.begin(); it != m_bookmarks.end(); ++it )
@@ -223,4 +245,68 @@ Smb4KBookmark *Smb4KBookmarkHandler::getBookmark( const QString &bookmark )
   return it != m_bookmarks.end() ? *it : NULL;
 }
 
+
+/****************************************************************************
+   Return the list of bookmarks
+****************************************************************************/
+
+const QValueList<Smb4KBookmark *> &Smb4KBookmarkHandler::getBookmarks()
+{
+  // Update the bookmarks:
+  update();
+
+  // Return the list of bookmarks:
+  return m_bookmarks;
+}
+
+
+
+/****************************************************************************
+   Update the bookmarks.
+****************************************************************************/
+
+void Smb4KBookmarkHandler::update()
+{
+  // If the user didn't pass the global Smb4KHostItem list, we are not able
+  // to update the bookmarks:
+  if ( !m_hosts )
+  {
+    return;
+  }
+
+  // Search the list of hosts for new IP addresses:
+  for ( QValueList<Smb4KBookmark *>::Iterator it = m_bookmarks.begin(); it != m_bookmarks.end(); ++it )
+  {
+    for ( QValueList<Smb4KHostItem *>::ConstIterator i = m_hosts->begin(); i != m_hosts->end(); ++i )
+    {
+      if ( QString::compare( (*it)->workgroup().lower(), (*i)->workgroup().lower() ) != 0 )
+      {
+        // Continue, if the workgroup is not the same:
+
+        continue;
+      }
+      else
+      {
+        if ( QString::compare( (*it)->host().lower(), (*i)->name().lower() ) != 0 )
+        {
+          // Continue if the host name is not the same:
+
+          continue;
+        }
+        else
+        {
+          // Set the IP address if it changed:
+          if ( !(*i)->ip().stripWhiteSpace().isEmpty() &&
+               QString::compare( (*it)->ip(), (*i)->ip() ) != 0 )
+          {
+            (*it)->setIP( (*i)->ip() );
+          }
+
+          break;
+        }
+      }
+    }
+  }
+}
+
 #include "smb4kbookmarkhandler.moc"
