@@ -36,6 +36,7 @@
 
 // Application includes
 #include "smb4kbookmark.h"
+#include "smb4knetworkitems.h"
 
 
 /**
@@ -53,27 +54,36 @@ class Smb4KBookmarkHandler : public QObject
     /**
      * The constructor.
      *
+     * @param hosts         A list of Smb4KHostItem items. This should be used
+     *                      to give the bookmark handler the list of known hosts,
+     *                      so it is able to update the bookmarks if necessary.
+     *                      In case a NULL pointer is passed, the bookmarks won't
+     *                      be updated.
+     *
      * @param parent        The parent of this object
      *
      * @param name          This object's name
      */
-    Smb4KBookmarkHandler( QObject *parent = 0, const char *name = 0 );
+    Smb4KBookmarkHandler( QValueList<Smb4KHostItem *> *hosts = 0,
+                          QObject *parent = 0,
+                          const char *name = 0 );
+
     /**
      * The destructor.
      */
     ~Smb4KBookmarkHandler();
+
     /**
-     * Writes a bookmark to the bookmark file. It is located under
-     * $HOME/.kde/share/apps/smb4k and stores, at the moment, only
-     * the name of the bookmark.
+     * This function writes a bookmark to the bookmark file.
      *
      * NOTE: This function is more or less a wrapper around the
      * writeBookmarList() function.
      *
-     * @param bookmark      An bookmark object that is to be written
-     *                      to the bookmark file
+     * @param bookmark      A bookmark that is to be written to
+     *                      the bookmark file
      */
     void addBookmark( Smb4KBookmark *bookmark );
+
     /**
      * This function writes a new list of bookmarks. The old list will be
      * deleted. It should be used, if you manipulated the list of bookmarks
@@ -84,13 +94,15 @@ class Smb4KBookmarkHandler : public QObject
      *                      to the bookmark file
      */
     void writeBookmarkList( const QValueList<Smb4KBookmark *> &list );
+
     /**
      * Get the list of bookmarks.
      *
      * @returns             The current list of bookmarks stored in the
      *                      bookmark file.
      */
-    const QValueList<Smb4KBookmark *> &getBookmarks() { return m_bookmarks; }
+    const QValueList<Smb4KBookmark *> &getBookmarks();
+
     /**
      * This function returns a single bookmark.
      *
@@ -100,7 +112,7 @@ class Smb4KBookmarkHandler : public QObject
      * @returns             The bookmark object that was searched or NULL if it
      *                      wasn't found.
      */
-    Smb4KBookmark *getBookmark( const QString &bookmark );
+    Smb4KBookmark *findBookmark( const QString &bookmark );
 
   signals:
     /**
@@ -113,6 +125,7 @@ class Smb4KBookmarkHandler : public QObject
      * The list of bookmarks.
      */
     QValueList<Smb4KBookmark *> m_bookmarks;
+
     /**
      * This function loads the list of bookmarks from the bookmarks file.
      * When it finishes, the bookmarksUpdated() signal is emitted. So, if you
@@ -120,5 +133,18 @@ class Smb4KBookmarkHandler : public QObject
      * connect a slot to that signal.
      */
     void loadBookmarks();
+
+    /**
+     * This lis is a pointer to a global list of all known hosts. It
+     * is used to update the bookmarks.
+     */
+    QValueList<Smb4KHostItem *> *m_hosts;
+
+    /**
+     * This function updates the data of the bookmarks, i.e. is searches for
+     * the host provided by m_hosts and sets the appropriate data, if
+     * necessary.
+     */
+    void update();
 };
 #endif
