@@ -36,12 +36,12 @@
 
 // KDE includes
 #include <klocale.h>
-#include <kapplication.h>
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
 #include <kiconloader.h>
 #include <klineedit.h>
 #include <kcombobox.h>
+#include <kdebug.h>
 
 #ifdef __FreeBSD__
 #include <sys/stat.h>
@@ -52,27 +52,47 @@
 // application specific includes
 #include "smb4kpasswordhandler.h"
 #include "smb4kdefs.h"
-#include "smb4kglobal.h"
-
-using namespace Smb4KGlobal;
+#include "smb4kerror.h"
 
 
-Smb4KPasswordHandler::Smb4KPasswordHandler( QObject *parent, const char *name )
-: QObject( parent, name )
+#ifndef __FreeBSD__
+Smb4KPasswordHandler::Smb4KPasswordHandler( KConfig *config, Smb4KHomesSharesHandler *handler, QObject *parent, const char *name )
+: QObject( parent, name ), m_config( config ), m_handler( handler )
+#else
+Smb4KPasswordHandler::Smb4KPasswordHandler( KConfig *config, Smb4KHomesSharesHandler *s_handler, Smb4KSambaOptionsHandler *o_handler, QObject *parent, const char *name )
+: QObject( parent, name ), m_config( config ), m_handler( s_handler ), m_options_handler( o_handler )
+#endif
 {
-  config()->setGroup( "Authentication" );
+  if ( !m_config )
+  {
+    kdFatal() << "Smb4KPasswordHandler: No KConfig object" << endl;
+  }
+
+  if ( !m_handler )
+  {
+    kdFatal() << "Smb4KPasswordHandler: No Smb4KHomesSharesHandler object" << endl;
+  }
+
+#ifdef __FreeBSD__
+  if ( !m_options_handler )
+  {
+    kdFatal() << "Smb4KPasswordHandler: No Smb4KSambaOptionsHandler object" << endl;
+  }
+#endif
 
   m_auth = NULL;
   m_dlg = NULL;
   m_wallet = NULL;
   m_temp_auth = NULL;
 
-  if ( !config()->hasKey( "Use Wallet" ) )
+  m_config->setGroup( "Authentication" );
+
+  if ( !m_config->hasKey( "Use Wallet" ) )
   {
-    config()->writeEntry( "Use Wallet", true );
+    m_config->writeEntry( "Use Wallet", true );
   }
 
-  config()->sync();
+  m_config->sync();
 }
 
 
@@ -85,17 +105,15 @@ Smb4KPasswordHandler::~Smb4KPasswordHandler()
 
   m_auth_list.clear();
 
-  m_wallet->disconnectApplication( KWallet::Wallet::NetworkWallet(), kapp->name() );
-  m_wallet->closeWallet( KWallet::Wallet::NetworkWallet(), false );
   delete m_wallet;
 }
 
 
 void Smb4KPasswordHandler::open_close_wallet()
 {
-  config()->setGroup( "Authentication" );
+  m_config->setGroup( "Authentication" );
 
-  if ( config()->readBoolEntry( "Use Wallet", true ) )
+  if ( m_config->readBoolEntry( "Use Wallet", true ) )
   {
     if ( !m_wallet || (m_wallet && !m_wallet->isOpen()) )
     {
@@ -114,7 +132,7 @@ void Smb4KPasswordHandler::open_close_wallet()
       }
       else
       {
-        KMessageBox::sorry( 0, i18n( "Opening the wallet failed! KWallet support will be disabled." ) );
+        Smb4KError::error( ERROR_OPENING_WALLET_FAILED, KWallet::Wallet::NetworkWallet(), QString::null );
 
         delete m_wallet;
         m_wallet = NULL;
@@ -122,9 +140,9 @@ void Smb4KPasswordHandler::open_close_wallet()
         // Do not remove the following line or the entry won't be written
         // properly. This seems to have to do with the transition to the global
         // config object.
-        config()->setGroup( "Authentication" );
-        config()->writeEntry( "Use Wallet", false );
-        config()->sync();
+        m_config->setGroup( "Authentication" );
+        m_config->writeEntry( "Use Wallet", false );
+        m_config->sync();
 
         import();
       }
@@ -134,9 +152,6 @@ void Smb4KPasswordHandler::open_close_wallet()
   {
     if ( m_wallet )
     {
-      m_wallet->disconnectApplication( KWallet::Wallet::NetworkWallet(), kapp->name() );
-      m_wallet->closeWallet( KWallet::Wallet::NetworkWallet(), false );
-
       delete m_wallet;
       m_wallet = NULL;
     }
@@ -144,14 +159,14 @@ void Smb4KPasswordHandler::open_close_wallet()
 }
 
 
-bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &host, const QString &share, int desc )
+bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &host, const QString &share, int desc, QWidget *parent, const char *name )
 {
   m_auth = readAuth( workgroup, host, share );
 
   // Set up the askpass dialog:
 
   m_dlg = new KDialogBase( KDialogBase::Plain, i18n( "Authentication" ), KDialogBase::Ok|KDialogBase::Cancel,
-                           KDialogBase::Ok, 0, "AskPassDlg", false, true );
+                           KDialogBase::Ok, parent, name, true, true );
 
   QFrame *frame = m_dlg->plainPage();
   QGridLayout *layout = new QGridLayout( frame );
@@ -190,7 +205,8 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
   }
 
   message.append( i18n( "Please enter authentication data for %1." ).arg(
-                  m_auth->share().stripWhiteSpace().isEmpty() ? m_auth->host() :
+                  m_auth->share().stripWhiteSpace().isEmpty() ?
+                  m_auth->host() :
                   "//"+m_auth->host()+"/"+m_auth->share() ) );
 
   QLabel *message_label = new QLabel( frame );
@@ -208,12 +224,14 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
   if ( QString::compare( m_auth->share(), "homes" ) != 0 )
   {
     user_edit = new KLineEdit( frame, "AskPassUserEdit" );
+    user_edit->setMinimumWidth( 200 );
     layout->addWidget( user_edit, 1, 1, 0 );
   }
   else
   {
     user_combo = new KComboBox( frame, "AskPassUserCombo" );
     user_combo->setEditable( true );
+    user_combo->setMinimumWidth( 200 );
     layout->addWidget( user_combo, 1, 1, 0 );
   }
 
@@ -225,8 +243,7 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
   layout->addWidget( pass_edit, 2, 1, 0 );
 
   m_dlg->setMainWidget( frame );
-  frame->setFixedWidth( 350 );
-  m_dlg->setFixedSize( m_dlg->sizeHint() );
+  m_dlg->setFixedSize( 350, m_dlg->sizeHint().height() );
   m_dlg->enableButtonOK( false );
 
   // Since we have to allow empty passwords, we will only connect
@@ -260,8 +277,7 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
   }
   else
   {
-    config()->setGroup( "Homes Shares" );
-    QStringList list = config()->readListEntry( host.upper() );
+    QStringList list = m_handler->homesUsers( host );
 
     user_combo->insertStringList( list );
     user_combo->setCurrentText( QString::null );
@@ -283,6 +299,8 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
 
       m_auth->setUser( user );
       m_auth->setPassword( pass );
+
+      writeAuth( m_auth );
     }
     else
     {
@@ -291,9 +309,9 @@ bool Smb4KPasswordHandler::askpass( const QString &workgroup, const QString &hos
 
       m_auth->setUser( user );
       m_auth->setPassword( pass );
-    }
 
-    writeAuth( m_auth );
+      writeAuth( m_auth );
+    }
 
     ok = true;
   }
@@ -314,8 +332,8 @@ Smb4KAuthInfo *Smb4KPasswordHandler::readAuth( const QString &workgroup, const Q
 {
   open_close_wallet();
 
-  config()->setGroup( "Authentication" );
-  bool use_default = config()->readBoolEntry( "Default Authentication" );
+  m_config->setGroup( "Authentication" );
+  bool use_default = m_config->readBoolEntry( "Default Authentication" );
 
   Smb4KAuthInfo *auth = new Smb4KAuthInfo( workgroup, host, share );
 
@@ -384,9 +402,9 @@ Smb4KAuthInfo *Smb4KPasswordHandler::readAuth( const QString &workgroup, const Q
     }
     else
     {
-      config()->setGroup( "Authentication" );
+      m_config->setGroup( "Authentication" );
 
-      if ( config()->readBoolEntry( "Default Authentication", false ) )
+      if ( m_config->readBoolEntry( "Default Authentication", false ) )
       {
         QStringList default_list = full_list.grep( "DEFAULT", false );
         QString user, pass;
@@ -413,8 +431,8 @@ Smb4KAuthInfo *Smb4KPasswordHandler::readAuth( const QString &workgroup, const Q
   }
   else
   {
-    config()->setGroup( "Authentication" );
-    bool remember = config()->readBoolEntry( "Remember Passwords", true );
+    m_config->setGroup( "Authentication" );
+    bool remember = m_config->readBoolEntry( "Remember Passwords", true );
 
     if ( remember && !m_auth_list.isEmpty() )
     {
@@ -511,9 +529,9 @@ void Smb4KPasswordHandler::writeAuth( Smb4KAuthInfo *auth )
   }
   else
   {
-    config()->setGroup( "Authentication" );
+    m_config->setGroup( "Authentication" );
 
-    if ( config()->readBoolEntry( "Remember Passwords", true ) )
+    if ( m_config->readBoolEntry( "Remember Passwords", true ) )
     {
       bool is_included = false;
 
@@ -611,14 +629,14 @@ void Smb4KPasswordHandler::writeDefaultAuth( Smb4KAuthInfo *auth )
 
 void Smb4KPasswordHandler::import()
 {
-  config()->setGroup( "Authentication" );
+  m_config->setGroup( "Authentication" );
 
   int key = 0;
 
-  if ( config()->hasKey( "Random Number" ) )
+  if ( m_config->hasKey( "Random Number" ) )
   {
-    key = config()->readNumEntry( "Random Number" );
-    config()->deleteEntry( "Random Number" );
+    key = m_config->readNumEntry( "Random Number" );
+    m_config->deleteEntry( "Random Number" );
   }
 
   QDir::setCurrent( locateLocal( "appdata", QString::null, KGlobal::instance() ) );
@@ -694,7 +712,7 @@ void Smb4KPasswordHandler::import()
     file.remove();
   }
 
-  config()->sync();
+  m_config->sync();
 }
 
 
@@ -757,6 +775,7 @@ void Smb4KPasswordHandler::slotReceivePassword( KProcess *, char *buffer, int bu
 
 void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
 {
+
   delete proc;
 
 #ifdef __FreeBSD__
@@ -790,9 +809,9 @@ void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
   if ( contents.find( "[default]" ) == contents.end() &&
        contents.find( "[DEFAULT]" ) == contents.end() )
   {
-    QMap<QString,QString> map = globalSMBOptions();
+    QMap<QString,QString> map = m_options_handler->globalSambaOptions();
     QString workgroup = map["workgroup"];
-    QString wins = winsServer();
+    QString wins = m_options_handler->winsServer();
 
     QStringList::Iterator it = contents.prepend( "[default]" );
     it++;
@@ -884,7 +903,7 @@ void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
   }
   else
   {
-    showCoreError( ERROR_WRITING_FILE, "~/.nsmbrc" );
+    Smb4KError::error( ERROR_WRITING_FILE, "~/.nsmbrc", QString::null );
 
     return;
   }
@@ -897,7 +916,7 @@ void Smb4KPasswordHandler::slotWritePassword( KProcess *proc )
   if ( (fd = open( path.ascii(), O_WRONLY )) == -1 )
   {
     int err_no = errno;
-    showCoreError( ERROR_OPENING_FILE, strerror( err_no ) );
+    Smb4KError::error( ERROR_OPENING_FILE, path, strerror( err_no ) );
 
     return;
   }
