@@ -38,9 +38,15 @@
 #include <kwallet.h>
 #include <kdialogbase.h>
 #include <kprocess.h>
+#include <kconfig.h>
 
 // application specific includes
 #include "smb4kauthinfo.h"
+#include "smb4khomesshareshandler.h"
+
+#ifdef __FreeBSD__
+#include "smb4ksambaoptionshandler.h"
+#endif
 
 
 
@@ -56,14 +62,47 @@ class Smb4KPasswordHandler : public QObject
   Q_OBJECT
 
   public:
+#ifndef __FreeBSD__
     /**
      * The constructor.
      *
+     * @param config        The KConfig object that should be used
+     *
+     * @param handler       The Smb4KHomesSharesHandler object that is needed to retrieve
+     *                      infos in case we need to look up authentication information for
+     *                      'homes' shares.
+     *
      * @param parent        The parent object.
      *
      * @param name          The name this class should carry.
      */
-    Smb4KPasswordHandler( QObject *parent = 0, const char *name = 0 );
+    Smb4KPasswordHandler( KConfig *config,
+                          Smb4KHomesSharesHandler *handler,
+                          QObject *parent = 0,
+                          const char *name = 0 );
+#else
+    /**
+     * The constructor (FreeBSD).
+     *
+     * @param config        The KConfig object that should be used
+     *
+     * @param s_handler     The Smb4KHomesSharesHandler object that is needed to retrieve
+     *                      infos in case we need to look up authentication information for
+     *                      'homes' shares.
+     *
+     * @param o_handler     The Smb4KSambaOptionsHandler object. It is needed to write the 
+     *                      ~/.nsmbrc file correctly.
+     *
+     * @param parent        The parent object.
+     *
+     * @param name          The name this class should carry.
+     */
+    Smb4KPasswordHandler( KConfig *config,
+                          Smb4KHomesSharesHandler *s_handler,
+                          Smb4KSambaOptionsHandler *o_handler,
+                          QObject *parent = 0,
+                          const char *name = 0 );
+#endif
 
     /**
      * The destructor.
@@ -83,9 +122,13 @@ class Smb4KPasswordHandler : public QObject
      *                      Values are taken from the Smb4KPasswordHandler::AskPass::Description
      *                      enumeration.
      *
+     * @param parent        The parent of this dialog.
+     *
+     * @param name          The name of this dialog.
+     *
      * @returns             TRUE if a new password has been supplied and FALSE otherwise.
      */
-    bool askpass( const QString &workgroup, const QString &host, const QString &share, int desc );
+    bool askpass( const QString &workgroup, const QString &host, const QString &share, int desc, QWidget *parent = 0, const char *name = 0 );
 
     /**
      * Returns the authentication data that was found for the defined share.
@@ -205,6 +248,16 @@ class Smb4KPasswordHandler : public QObject
      */
     Smb4KAuthInfo *m_temp_auth;
 
+    /**
+     * The KConfig object
+     */
+    KConfig *m_config;
+
+    /**
+     * The Smb4KHomesSharesHandler object
+     */
+    Smb4KHomesSharesHandler *m_handler;
+
 #ifdef __FreeBSD__
 
     /**
@@ -215,16 +268,22 @@ class Smb4KPasswordHandler : public QObject
      *                      to ~/.nsmbrc file.
      */
     void writeToSMBConfFile( Smb4KAuthInfo *auth );
+    
     /**
      * FreeBSD specific: Holds the authentication information that should
      * be written to ~/.nsmbrc.
      */
     Smb4KAuthInfo m_nsmbrc_auth;
+    
     /**
      * The buffer for the process that encrypts the password.
      */
     QString m_buffer;
-
+    
+    /**
+     * The Smb4KSambaOptionsHandler object.
+     */
+    Smb4KSambaOptionsHandler *m_options_handler;
 #endif
 
 };
