@@ -28,11 +28,9 @@
 #include <qfile.h>
 
 // KDE includes
-#include <klocale.h>
-#include <kapplication.h>
-#include <kmessagebox.h>
 #include <kurl.h>
 #include <kfileitem.h>
+#include <kdebug.h>
 
 // system includes
 #include <stdlib.h>
@@ -40,11 +38,10 @@
 // application specific includes
 #include "smb4kprint.h"
 #include "smb4kdefs.h"
+#include "smb4kerror.h"
 #include "smb4kglobal.h"
-
 using namespace Smb4KGlobal;
 
-bool retry = false;
 
 
 Smb4KPrint::Smb4KPrint( QObject *parent, const char *name ) : QObject( parent, name )
@@ -54,8 +51,6 @@ Smb4KPrint::Smb4KPrint( QObject *parent, const char *name ) : QObject( parent, n
 
   m_info = NULL;
 
-  m_password_handler = new Smb4KPasswordHandler( this, "PrintPasswordHandler" );
-
   m_working = false;
 
   connect( m_proc, SIGNAL( receivedStdout( KProcess *, char *, int ) ),
@@ -73,69 +68,6 @@ Smb4KPrint::~Smb4KPrint()
 }
 
 
-/****************************************************************************
-   Initializes the printing of a file. This function looks for the mime
-   type and takes care of the further processing.
-****************************************************************************/
-
-
-bool Smb4KPrint::init( Smb4KPrintInfo *info )
-{
-  m_working = true;
-
-  config()->setGroup( "Programs" );
-  QString enscript = config()->readPathEntry( "enscript" );
-  QString dvips = config()->readPathEntry( "dvips" );
-
-  bool ok = true;
-
-  if ( QFile::exists( info->path() ) )
-  {
-    // We want to determine the mimetype of the incoming
-    // file. Let's use KFileItem for this, because KFileMetaInfo
-    // does not work very reliable.
-    KURL u;
-    u.setPath( info->path() );
-
-    KFileItem item = KFileItem( KFileItem::Unknown, KFileItem::Unknown, u, false );
-
-    if ( QString::compare( item.mimetype(), "application/postscript" ) == 0 ||
-         QString::compare( item.mimetype(), "application/pdf" ) == 0 ||
-         item.mimetype().startsWith( "image" ) )
-    {
-      m_info = info;
-      startPrinting();
-    }
-    else if ( QString::compare( item.mimetype(), "application/x-dvi" ) == 0 && !dvips.isEmpty() )
-    {
-      m_info = info;
-      convertDVIToPS();
-    }
-    else if ( ( item.mimetype().startsWith( "text" ) || item.mimetype().startsWith( "message" ) ) && !enscript.isEmpty() )
-    {
-      m_info = info;
-      convertTextToPS();
-    }
-    else
-    {
-      showCoreError( ERROR_MIMETYPE_NOT_SUPPORTED, item.mimetype() );
-    }
-  }
-  else
-  {
-    showCoreError( ERROR_FILE_NOT_FOUND, info->path() );
-    ok = false;
-  }
-
-  if ( !m_info )
-  {
-    delete info;
-  }
-
-  return ok;
-}
-
-
 /****************************************************************************
    Aborts the current process.
 ****************************************************************************/
@@ -150,12 +82,93 @@ void Smb4KPrint::abort()
 
 
 /****************************************************************************
-   Starts the actual printing.
+   Start the printing.
 ****************************************************************************/
 
-void Smb4KPrint::startPrinting()
+bool Smb4KPrint::print( Smb4KPrintInfo *info )
 {
-  Smb4KAuthInfo *auth = m_password_handler->readAuth( m_info->workgroup(), m_info->host(), m_info->printer() );
+  // Do nothing if we receive a NULL pointer:
+  if ( !info )
+  {
+    return false;
+  }
+
+  m_working = true;
+  m_info = info;
+
+  // Get the installed extra programs:
+  config()->setGroup( "Programs" );
+  QString enscript = config()->readPathEntry( "enscript" );
+  QString dvips = config()->readPathEntry( "dvips" );
+
+  // Start processing the file:
+  if ( QFile::exists( m_info->path() ) )
+  {
+    // Determine the mimetype of the file:
+    KURL url;
+    url.setPath( m_info->path() );
+
+    KFileItem file_item = KFileItem( KFileItem::Unknown, KFileItem::Unknown, url, false );
+
+    kdDebug() << file_item.mimetype() << endl;
+
+    return true;
+
+    if ( QString::compare( file_item.mimetype(), "application/postscript" ) == 0 ||
+         QString::compare( file_item.mimetype(), "application/pdf" ) == 0 ||
+         file_item.mimetype().startsWith( "image" ) )
+    {
+      setDeviceURI();
+      printNormal();
+    }
+    else if ( QString::compare( file_item.mimetype(), "application/x-dvi" ) == 0 &&
+              !dvips.isEmpty() )
+    {
+      setDeviceURI();
+      printDVI();
+    }
+    else if ( (file_item.mimetype().startsWith( "text" ) ||
+               file_item.mimetype().startsWith( "message" )) &&
+              !enscript.isEmpty() )
+    {
+      setDeviceURI();
+      printText();
+    }
+    else
+    {
+      Smb4KError::information( INFO_MIMETYPE_NOT_SUPPORTED, file_item.mimetype() );
+
+      delete m_info;
+      m_info = NULL;
+
+      m_working = false;
+      emit state( PRINT_STOP );
+      return false;
+    }
+  }
+  else
+  {
+    Smb4KError::error( ERROR_FILE_NOT_FOUND, m_info->path() );
+
+    delete m_info;
+    m_info = NULL;
+
+    m_working = false;
+    emit state( PRINT_STOP );
+    return false;
+  }
+
+  return true;
+}
+
+
+/****************************************************************************
+   Sets the device URI
+****************************************************************************/
+
+void Smb4KPrint::setDeviceURI()
+{
+  Smb4KAuthInfo *auth = passwordHandler()->readAuth( m_info->workgroup(), m_info->host(), m_info->printer() );
 
   QString uri;
 
@@ -187,161 +200,90 @@ void Smb4KPrint::startPrinting()
 
   m_proc->setEnvironment( "DEVICE_URI", uri );
 
-  // FIXME: If smbspool starts to use argv[1] and argv[5]
-  // we have to implement them!
-  QString command = QString( "smbspool 111 %1 \"Smb4K print job\" %2 \"\" %3" ).arg( getenv( "USER" ) ).arg( m_info->copies() ).arg( KProcess::quote( m_info->path() ) );
-
   delete auth;
+}
+
+
+/****************************************************************************
+   Do normal printing.
+****************************************************************************/
+
+void Smb4KPrint::printNormal()
+{
+  QString command;
+
+  command.append( "smbspool 111 "+QString( getenv( "USER" ) ) );
+  command.append( " \"Smb4K print job\" "+QString( "%1" ).arg( m_info->copies() ) );
+  command.append( " \"\" "+KProcess::quote( m_info->path() ) );
 
   *m_proc << command;
 
-  startProcess( Print );
-}
-
-
-/****************************************************************************
-   Converts DVI files to PS using dvips.
-****************************************************************************/
-
-void Smb4KPrint::convertDVIToPS()
-{
-  QString path = m_info->path().section( "/", 0, -2 );
-  QString file = m_info->path().section( "/", -1, -1 );
-
-  *m_proc << "cd "+KProcess::quote( path )+" && dvips -P pdf -o /tmp/smb4k_print_$USER.ps "+KProcess::quote( file );
-  startProcess( Convert );
-}
-
-
-/****************************************************************************
-   Converts text files to PS using enscript.
-****************************************************************************/
-
-void Smb4KPrint::convertTextToPS()
-{
-  *m_proc << "enscript -1 -B --ps-level=2 -o /tmp/smb4k_print_$USER.ps "+KProcess::quote( m_info->path() );
-  startProcess( Convert );
-}
-
-
-/****************************************************************************
-   Starts any process of this class.
-****************************************************************************/
-
-void Smb4KPrint::startProcess( int internal_state )
-{
-  m_internal_state = internal_state;
-  m_buffer = QString::null;
-
-  if ( m_internal_state == Print )
-  {
-    emit state( PRINT_SEND_FILE );
-  }
-  else
-  {
-    emit state( PRINT_CONVERT_FILE );
-  }
+  emit state( PRINT_START );
 
   m_proc->start( KProcess::NotifyOnExit, KProcess::AllOutput );
 }
 
 
 /****************************************************************************
-   If the KProcess ended, this function takes care of the further
-   processing.
+   Print DVI files.
 ****************************************************************************/
 
-void Smb4KPrint::endProcess()
+void Smb4KPrint::printDVI()
 {
-  switch ( m_internal_state )
-  {
-    case Print:
-      endPrintProcess();
-      break;
-    case Convert:
-      endConversionProcess();
-      break;
-    default:
-      break;
-  }
+  // The temporary file.
+  QString temp_file = tempDir()+"/smb4k_print.ps";
 
-  m_internal_state = Idle;
-  m_proc->clearArguments();
+  QString command;
 
-  if ( !retry )
-  {
-    delete m_info;
-    m_info = NULL;
-  }
+  // First we need the conversion:
+  command.append( "cd "+KProcess::quote( m_info->path().section( "/", 0, -2 ) )+" && " );
+  command.append( "dvips -P pdf -o "+temp_file+" "+KProcess::quote( m_info->path().section( "/", -1, -1 ) )+" && " );
 
-  m_working = false;
-  emit state( PRINT_STOP );
+  // The actual print command:
+  command.append( "smbspool 111 "+QString( getenv( "USER" ) ) );
+  command.append( " \"Smb4K print job\" "+QString( "%1" ).arg( m_info->copies() ) );
+  command.append( " \"\" "+KProcess::quote( temp_file )+" && " );
+
+  // Clean up:
+  command.append( "rm -f "+temp_file );
+
+  *m_proc << command;
+
+  emit state( PRINT_START );
+
+  m_proc->start( KProcess::NotifyOnExit, KProcess::AllOutput );
 }
 
 
 /****************************************************************************
-   Post-processing of any print job.
+   Print text files.
 ****************************************************************************/
 
-void Smb4KPrint::endPrintProcess()
+void Smb4KPrint::printText()
 {
-  if ( m_buffer.contains( "NT_STATUS", true ) != 0 )
-  {
-    if ( m_buffer.contains( "NT_STATUS_ACCESS_DENIED" ) != 0 || m_buffer.contains( "NT_STATUS_LOGON_FAILURE" ) != 0 )
-    {
-      int state = Smb4KPasswordHandler::None;
+  // The temporary file.
+  QString temp_file = tempDir()+"/smb4k_print.ps";
 
-      if ( m_buffer.contains( "NT_STATUS_ACCESS_DENIED" ) != 0 )
-      {
-        state = Smb4KPasswordHandler::AccessDenied;
-      }
-      else if (m_buffer.contains( "NT_STATUS_LOGON_FAILURE" ) != 0  )
-      {
-        state = Smb4KPasswordHandler::LogonFailure;
-      }
+  QString command;
 
-      if ( m_password_handler->askpass( m_info->workgroup(), m_info->host(), m_info->printer(), state ) )
-      {
-        retry = true;
-        QTimer::singleShot( 50, this, SLOT( slotRetry() ) );
-      }
-    }
-    else
-    {
-      showCoreError( ERROR_PRINTING, m_buffer );
+  // Conversion:
+  command.append( "enscript --columns=1 --no-header --ps-level=2 " );
+  command.append( "-o "+KProcess::quote( temp_file )+" " );
+  command.append( KProcess::quote( m_info->path() )+ " && " );
 
-      // Clean up:
-      QFile::remove( "/tmp/smb4k_print_"+QString( "%1" ).arg( getenv( "USER" ) )+".ps" );
-    }
-  }
-  else
-  {
-    // Clean up:
-    QFile::remove( "/tmp/smb4k_print_"+QString( "%1" ).arg( getenv( "USER" ) )+".ps" );
-  }
-}
+  // The actual print command:
+  command.append( "smbspool 111 "+QString( getenv( "USER" ) ) );
+  command.append( " \"Smb4K print job\" "+QString( "%1" ).arg( m_info->copies() ) );
+  command.append( " \"\" "+KProcess::quote( temp_file )+" && " );
 
+  // Clean up:
+  command.append( "rm -f "+temp_file );
 
-/****************************************************************************
-   Ends the conversion process.
-****************************************************************************/
+  *m_proc << command;
 
-void Smb4KPrint::endConversionProcess()
-{
-  if ( m_buffer.contains( "command not found", true ) != 0 )
-  {
-    QString missing = m_buffer.section( ":", -2, -2 ).section( ":", -1, -1 ).stripWhiteSpace();
+  emit state( PRINT_START );
 
-    showCoreError( ERROR_COMMAND_NOT_FOUND, missing );
-  }
-  else
-  {
-    // Set the URL to the newly created PS file:
-    m_info->setPath( "/tmp/smb4k_print_"+QString( "%1" ).arg( getenv( "USER" ) )+".ps" );
-
-    retry = true;
-    QTimer::singleShot( 50, this, SLOT( slotRetry() ) );
-  }
+  m_proc->start( KProcess::NotifyOnExit, KProcess::AllOutput );
 }
 
 
@@ -368,14 +310,62 @@ void Smb4KPrint::slotReceivedStderr( KProcess *, char *buf, int len )
 
 void Smb4KPrint::slotProcessExited( KProcess * )
 {
-  endProcess();
+  bool retry = false;
+
+  if ( m_buffer.contains( "NT_STATUS", true ) != 0 ||
+       m_buffer.contains( "enscript", true ) != 0  ||
+       m_buffer.contains( "dvips", true ) != 0 )
+  {
+    if ( m_buffer.contains( "NT_STATUS_ACCESS_DENIED" ) != 0 || m_buffer.contains( "NT_STATUS_LOGON_FAILURE" ) != 0 )
+    {
+      int state = Smb4KPasswordHandler::None;
+
+      if ( m_buffer.contains( "NT_STATUS_ACCESS_DENIED" ) != 0 )
+      {
+        state = Smb4KPasswordHandler::AccessDenied;
+      }
+      else if (m_buffer.contains( "NT_STATUS_LOGON_FAILURE" ) != 0  )
+      {
+        state = Smb4KPasswordHandler::LogonFailure;
+      }
+
+      if ( passwordHandler()->askpass( m_info->workgroup(), m_info->host(), m_info->printer(), state ) )
+      {
+        retry = true;
+        QTimer::singleShot( 50, this, SLOT( slotRetry() ) );
+      }
+    }
+    else
+    {
+      Smb4KError::error( ERROR_PRINTING, m_info->path(), m_buffer );
+
+      // Clean up:
+      QFile::remove( QString( "%1/smb4k_print.ps" ).arg( tempDir() ) );
+    }
+  }
+  else
+  {
+    // Clean up:
+    QFile::remove( QString( "%1/smb4k_print.ps" ).arg( tempDir() ) );
+  }
+
+  m_proc->clearArguments();
+
+  if ( !retry )
+  {
+    delete m_info;
+    m_info = NULL;
+  }
+
+  m_working = false;
+  emit state( PRINT_STOP );
 }
 
 
 void Smb4KPrint::slotRetry()
 {
-  retry = false;
-  startPrinting();
+  print( m_info );
 }
 
+
 #include "smb4kprint.moc"
