@@ -33,6 +33,9 @@
 // Qt includes
 #include <qstring.h>
 
+// application specific includes
+#include "smb4knetworkitems.h"
+
 /**
  * This class provides a container that holds all the info
  * that is needed to print a file.
@@ -43,8 +46,16 @@ class Smb4KPrintInfo
   public:
     /**
      * The constructor.
+     *
+     * @param item          The Smb4KShareItem that represents the printer
+     *
+     * @param ip            The IP address of the host
+     *
+     * @param filepath      The path of the file to print
+     *
+     * @param copies        The number of copies
      */
-    Smb4KPrintInfo( const QString &workgroup, const QString &host, const QString &ip, const QString &printer, const QString &filepath, int copies = 1 );
+    Smb4KPrintInfo( Smb4KShareItem *item, const QString &ip, const QString &filepath = QString::null, int copies = 1 );
     /**
      * Empty constructor.
      */
@@ -56,23 +67,23 @@ class Smb4KPrintInfo
     /**
      * Returns the path of the file.
      */
-    QString path() const;
+    const QString &path() const;
     /**
      * Returns the host where the printer is located.
      */
-    QString host() const;
+    const QString &host() const;
     /**
      * Returns the workgroup in which the host located.
      */
-    QString workgroup() const;
+    const QString &workgroup() const;
     /**
      * Returns the name of the printer.
      */
-    QString printer() const;
+    const QString &printer() const;
     /**
      * Returns the IP address of the host.
      */
-    QString ip() const;
+    const QString &ip() const;
     /**
      * Sets the path to the file to print.
      */
@@ -85,6 +96,10 @@ class Smb4KPrintInfo
      * Sets the number of copies.
      */
     void setCopies( int num );
+    /**
+     * Returns the comment.
+     */
+    const QString &comment() const;
 
   private:
     /**
@@ -111,6 +126,10 @@ class Smb4KPrintInfo
      * Holds the number of copies the user wants to have.
      */
     int m_copies;
+    /**
+     * The comment
+     */
+    QString m_comment;
 };
 
 #endif
