@@ -34,6 +34,7 @@
 
 // Qt includes
 #include <qstring.h>
+#include <qcstring.h>
 
 // application specific includes
 #include "smb4kuser.h"
@@ -63,6 +64,7 @@ class Smb4KShare
      * @param broken      Determines whether the share is broken. "Broken" means that the share is unaccessible.
      */
     Smb4KShare( const QString &name, const QString &path, const QString &filesystem, const int uid = 0, const int gid = 0, bool broken = false );
+
     /**
      * The constructor for CIFS shares. It does not take the UID and GID, but the
      * user name with which the login was done.
@@ -79,34 +81,40 @@ class Smb4KShare
      *
      * @param broken      Determines whether the share is broken. "Broken" means that the share is unaccessible.
      */
-    Smb4KShare( const QString &name, const QString &path, const QString &filesystem, const QString &username, bool foreign, bool broken = false );
+    Smb4KShare( const QString &name, const QString &path, const QString &filesystem, const QString &username, bool foreign = false, bool broken = false );
+
     /**
      * Empty constructor.
      */
     Smb4KShare() {}
+
     /**
      * Copy constructor.
      *
      * @param share       The share that is to be copied.
      */
     Smb4KShare( const Smb4KShare &share );
+
     /**
      * The destructor
      */
     ~Smb4KShare();
+
     /**
      * Returns the name of the share as it has been gathered by the mounter.
      *
      * @returns           The name of the share.
      */
-    QString name() const;
+    const QString &name() const;
+
     /**
      * Returns the mount point aka path of the share as it has been gathered
-     * by the mounter.
+     * by the mounter. This is a C-type string.
      *
      * @returns           The path of the share.
      */
-    QString path() const;
+    const QCString &path() const;
+
     /**
      * This function returns the canonical path of the share. In contrast to
      * Smb4KShare::path(), it will return the absolute path without symlinks. However,
@@ -115,46 +123,65 @@ class Smb4KShare
      *
      * @returns           Returns the canonical path of the share.
      */
-    QString canonicalPath() const;
+    const QCString canonicalPath() const;
+
     /**
      * Returns the UID of the mounted share.
      */
     int uid() const;
+
     /**
      * Returns the GID of the mounted share.
      */
     int gid() const;
+
     /**
      * Returns the name of the user of the share.
      */
-    QString user() const;
+    const QString &user() const;
+
     /**
      * Returns the name of the group of the share.
      */
-    QString group() const;
+    const QString &group() const;
+
     /**
      * Returns the file system of the share.
      */
-    QString filesystem() const;
+    const QString &filesystem() const;
+
     /**
      * Returns the CIFS login (user name).
      */
-    QString cifsLogin() const;
+    const QString &cifsLogin() const;
+
     /**
      * Is TRUE if the share is/seems to be mounted by another
      * user.
+     *
+     * @returns           TRUE if another user mounted the share and FALSE otherwise.
      */
     bool isForeign() const;
+
+    /**
+     * This function sets the share to be foreign.
+     *
+     * @param foreign     TRUE if share is foreign and FALSE otherwise.
+     */
+    void setForeign( bool foreign );
+
     /**
      * Returns TRUE if the share is broken and FALSE otherwise.
      */
     bool isBroken() const;
+
     /**
      * Sets the share to be broken.
      *
      * @param broken      TRUE if the share is broken and FALSE otherwise.
      */
     void setBroken( bool broken );
+
     /**
      * This function sets the value of the total disk usage. The value has to
      * be provided in kilobytes. If the disk usage could not be determined,
@@ -163,6 +190,7 @@ class Smb4KShare
      * @param total       The total disk usage in kB.
      */
     void setTotalDiskSpace( double total );
+
     /**
      * This function sets the value of the free space on the share. The value
      * has to be provided in kilobytes. If the free space could not be determined,
@@ -171,18 +199,21 @@ class Smb4KShare
      * @param free        The free disk space in kB.
      */
     void setFreeDiskSpace( double free );
+
     /**
      * This function returns the total disk space of the share.
      *
      * @returns           The total disk space in kB.
      */
     double totalDiskSpace() const;
+
     /**
      * This function returns the free disk space available on the share in kB.
      *
      * @returns           the free disk space in kB.
      */
     double freeDiskSpace() const;
+
     /**
      * This function returns the percentage of used disk space on the
      * share.
@@ -193,10 +224,29 @@ class Smb4KShare
 
 
   private:
+    /**
+     * The name of the share.
+     */
     QString m_name;
-    QString m_path;
+
+    /**
+     * The mount point / path of the share
+     */
+    QCString m_path;
+
+    /**
+     * The filesystem string
+     */
     QString m_filesystem;
+
+    /**
+     * The user and group
+     */
     Smb4KUser m_userinfo;
+
+    /**
+     * The CIFS login name
+     */
     QString m_cifs_login;
     bool m_foreign_mount;
     bool m_broken;
