@@ -43,6 +43,7 @@
 // application specific includes
 #include "smb4ksynchronizer.h"
 #include "smb4kdefs.h"
+#include "smb4kerror.h"
 #include "smb4kglobal.h"
 
 using namespace Smb4KGlobal;
@@ -85,7 +86,7 @@ Smb4KSynchronizer::~Smb4KSynchronizer()
    Synchronizes a share with a local copy or vice versa.
 ****************************************************************************/
 
-void Smb4KSynchronizer::synchronize( Smb4KShare *share )
+void Smb4KSynchronizer::synchronize( Smb4KShare *share, QWidget *parent )
 {
   config()->setGroup( "Programs" );
 
@@ -97,7 +98,14 @@ void Smb4KSynchronizer::synchronize( Smb4KShare *share )
   config()->setGroup( "Rsync" );
   QString default_prefix = config()->readEntry( "Prefix", QDir::homeDirPath()+"/smb4k_sync/" );
 
-  m_chooser = new KDialogBase( KDialogBase::Plain, i18n( "Synchronization" ),KDialogBase::User1|KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, 0, "SynchronizationDialog", false, true );
+  m_chooser = new KDialogBase( KDialogBase::Plain, i18n( "Synchronization" ),
+                               KDialogBase::User1|KDialogBase::Ok|KDialogBase::Cancel,
+                               KDialogBase::Ok,
+                               parent,
+                               "SynchronizationDialog",
+                               false,
+                               true );
+
   m_chooser->setButtonGuiItem( KDialogBase::User1, KGuiItem( i18n( "Swap Paths" ), QString::null, i18n( "Swap source and destination" ) ) );
   connect( m_chooser, SIGNAL( user1Clicked() ),
            this,      SLOT( slotSwapURLs() ) );
@@ -156,7 +164,14 @@ void Smb4KSynchronizer::synchronize( Smb4KShare *share )
   *m_proc << command;
 
   // Set up the progress dialog.
-  m_dialog = new KDialogBase( KDialogBase::Plain, i18n( "Synchronization" ), KDialogBase::Cancel, KDialogBase::Cancel, 0, "SyncProgDlg", false, true );
+  m_dialog = new KDialogBase( KDialogBase::Plain,
+                              i18n( "Synchronization" ),
+                              KDialogBase::Cancel,
+                              KDialogBase::Cancel,
+                              parent,
+                              "SynchronizationProgressDialog",
+                              false,
+                              true );
 
   QFrame *frame = m_dialog->plainPage();
   QGridLayout *layout = new QGridLayout( frame );
@@ -190,9 +205,14 @@ void Smb4KSynchronizer::synchronize( Smb4KShare *share )
 
   m_dialog->setFixedSize( m_dialog->sizeHint() );
 
+  // Never change this, because otherwise we will start the
+  // process AFTER the dialog has been closed:
   m_dialog->show();
 
-  m_proc->start( KProcess::NotifyOnExit, KProcess::AllOutput );
+  // Use KProcess::OwnGroup instead of KProcess::NotifyOnExit here, because
+  // this garantees that the process is indeed killed when using abort().
+  // See KProcess docs for further information.
+  m_proc->start( KProcess::OwnGroup, KProcess::AllOutput );
 }
 
 
@@ -297,7 +317,10 @@ void Smb4KSynchronizer::abort()
 
 void Smb4KSynchronizer::slotProcessExited( KProcess * )
 {
-  ((KProgress *)m_dialog->child( "SyncProgDlgProgTotal", "KProgress", true ))->setValue( 100 );
+  if ( !cancel )
+  {
+    ((KProgress *)m_dialog->child( "SyncProgDlgProgTotal", "KProgress", true ))->setValue( 100 );
+  }
 
   m_proc->clearArguments();
 
@@ -311,7 +334,6 @@ void Smb4KSynchronizer::slotProcessExited( KProcess * )
 }
 
 
-
 void Smb4KSynchronizer::slotReceivedStdout( KProcess *, char *buf, int len )
 {
   m_buffer = QString::fromLocal8Bit( buf, len );
@@ -378,7 +400,7 @@ void Smb4KSynchronizer::slotReceivedStdout( KProcess *, char *buf, int len )
 
       if ( f )
       {
-        f->setText( QString( "%1 of %2" ).arg( files, m_total_files ) );
+        f->setText( i18n( "%1 of %2" ).arg( files, m_total_files ) );
       }
     }
 
@@ -441,7 +463,7 @@ void Smb4KSynchronizer::slotReceivedStderr( KProcess *, char *buf, int len )
     // especially if you try to synchronize a local copy with a remote
     // share that is mounted read-only.
     abort();
-    showCoreError( ERROR_SYNCHRONIZING, error_message );
+    Smb4KError::error( ERROR_SYNCHRONIZING, QString::null, error_message );
   }
   else
   {
