@@ -7,12 +7,20 @@
  ***************************************************************************/
 
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   This program is distributed in the hope that it will be useful, but   *
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
+ *   General Public License for more details.                              *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
+ *   MA  02111-1307 USA                                                    *
  ***************************************************************************/
 
 #ifndef SMB4KAUTHOPTIONS_H
@@ -38,7 +46,7 @@
 class Smb4KAuthOptions : public QWidget
 {
   friend class Smb4KOptionsDlg;
-  
+
   Q_OBJECT
   public:
     /**
@@ -50,21 +58,36 @@ class Smb4KAuthOptions : public QWidget
      */
     ~Smb4KAuthOptions();
 
+  protected slots:
+    /**
+     * This slot manages the toggling of the KWallet support button in the
+     * Authentication tab.
+     *
+     * @param state         The toggle state
+     */
+    void slotKWalletButtonState( int state );
+    /**
+     * This slot manages the toggling of the default authentication button.
+     *
+     * @param state        The toggle state
+     */
+    void slotDefaultAuthButtonState( int state );
+
   private:
-    /** 
+    /**
      * Default authentication
      */
     QCheckBox *m_defaultAuth;
-    /** 
+    /**
      * Default login
      */
     KLineEdit *m_defaultLogin;
-    /** 
+    /**
      * Default password
      */
     KLineEdit *m_defaultPasswd;
-    /** 
-     * The container for m_defaultLogin and m_defaultPasswd. 
+    /**
+     * The container for m_defaultLogin and m_defaultPasswd.
      */
     QWidget *m_authWidget;
     /**
@@ -76,7 +99,7 @@ class Smb4KAuthOptions : public QWidget
      */
     QVGroupBox *m_login_box;
     /**
-     * Determines if the passwords should be stored temporarily 
+     * Determines if the passwords should be stored temporarily
      * if KWallet support is switched off or if they shouldn't.
      */
     QCheckBox *m_remember_passwords;
