@@ -45,7 +45,7 @@
  */
 
 class Smb4KBrowserWidget;
-class Smb4KBrowserWidgetItem : public QListViewItem
+class Smb4KBrowserWidgetItem : public KListViewItem
 {
   public:
     /**
@@ -165,6 +165,16 @@ class Smb4KBrowserWidgetItem : public QListViewItem
      */
     void update( Smb4KShareItem *item );
 
+    /**
+     * This enumeration enumerates the columns of the item.
+     */
+    enum Columns{ Network = 0, Type = 1, IP = 2, Comment = 3 };
+
+    /**
+     * This function (re-)sets the icon of the network item.
+     */
+    void setIcon();
+
   protected:
     /**
      * Reimplemented from QListViewItem.
@@ -203,11 +213,6 @@ class Smb4KBrowserWidgetItem : public QListViewItem
      * This is the icon (desktop size) of this browser widget item
      */
     QPixmap m_desktop_icon;
-
-    /**
-     * This enumeration enumerates the columns of the item.
-     */
-    enum Columns{ Network = 0, Type = 1, IP = 2, Comment = 3 };
 };
 
 #endif
