@@ -80,7 +80,9 @@ void Smb4KNetworkItemTooltip::setupTooltip( Smb4KBrowserWidgetItem *item )
       QString mbl_entry = item->workgroupItem()->pseudo() ? i18n( "Pseudo master browser:" ) : i18n( "Master browser:" );
       QLabel *mb_label = new QLabel( mbl_entry, this );
 
-      QString mb_entry = item->workgroupItem()->ip().isEmpty() ? item->workgroupItem()->master() : item->workgroupItem()->master() + " ("+item->workgroupItem()->ip()+")";
+      QString mb_entry = item->workgroupItem()->ip().isEmpty() ?
+                         (item->workgroupItem()->master().isEmpty() ? i18n( "Unknown" ) : item->workgroupItem()->master()) :
+                         item->workgroupItem()->master() + " ("+item->workgroupItem()->ip()+")";
       QLabel *master = new QLabel( mb_entry, this, "MasterBrowser" );
 
       m_layout->addWidget( w_label, 0, 1, 0 );
@@ -195,13 +197,20 @@ void Smb4KNetworkItemTooltip::setupTooltip( Smb4KBrowserWidgetItem *item )
 }
 
 
-void Smb4KNetworkItemTooltip::showTip()
+void Smb4KNetworkItemTooltip::showTip( const QPoint &pos )
 {
-  if ( !m_item )
+  if ( !m_item || isShown() )
   {
     return;
   }
 
+  // Update the position where the tooltip
+  // should be shown.
+  if ( m_pos != pos )
+  {
+    m_pos = pos;
+  }
+
   update();
 
   adjustSize();
@@ -210,12 +219,20 @@ void Smb4KNetworkItemTooltip::showTip()
 
   if ( m_pos.x() + width() > d->width() )
   {
-    m_pos.setX( m_pos.x() - width() );
+    m_pos.setX( m_pos.x() - width() - 5 );
+  }
+  else
+  {
+    m_pos.setX( m_pos.x() + 5 );
   }
 
   if ( m_pos.y() + height() > d->height() )
   {
-    m_pos.setY( m_pos.y() - height() );
+    m_pos.setY( m_pos.y() - height() - 5 );
+  }
+  else
+  {
+    m_pos.setY( m_pos.y() + 5 );
   }
 
   setGeometry( m_pos.x(), m_pos.y(), width(), height() );
