@@ -44,18 +44,24 @@ Smb4KNetworkOptions::Smb4KNetworkOptions( QWidget *parent, const char *name ) :
   //
   QButtonGroup *browsebox = new QButtonGroup( 1, Qt::Horizontal, i18n( "Browse List" ), this );
 
-  m_useNMB = new QRadioButton( i18n( "Scan the network to retrieve the browse list" ), browsebox );
-  m_useMaster = new QRadioButton( i18n( "Query the current workgroup master browser to retrieve the browse list" ), browsebox );
+  m_useNMB = new QRadioButton( i18n( "Scan the network neighborhood for available workgroups and domains" ), browsebox );
+  m_useMaster = new QRadioButton( i18n( "Query the current workgroup master browser" ), browsebox );
 
-  QWidget *masterWidget = new QWidget( browsebox );
-  QGridLayout *masterLayout = new QGridLayout( masterWidget );
-  m_useHost = new QRadioButton( i18n( "Query this master browser to retrieve the browse list:" ), masterWidget );
-  m_hostName = new KLineEdit( masterWidget );
+  QWidget *buttonWidget = new QWidget( browsebox );
+  QGridLayout *buttonLayout = new QGridLayout( buttonWidget );
+  buttonLayout->setSpacing( 5 );
+  m_useHost = new QRadioButton( i18n( "Query this master browser:" ), buttonWidget );
+  m_hostName = new KLineEdit( buttonWidget );
   m_hostName->setMinimumWidth( 150 );
-  masterLayout->addWidget( m_useHost, 0, 0, 0 );
-  masterLayout->addWidget( m_hostName, 0, 1, 0 );
+  m_useIPScan = new QRadioButton( i18n( "Broadcast addresses for IP scan:" ), buttonWidget );
+  m_ipBroadcastAddresses = new KLineEdit( buttonWidget );
+  buttonLayout->addWidget( m_useHost, 0, 0, 0 );
+  buttonLayout->addWidget( m_hostName, 0, 1, 0 );
+  buttonLayout->addWidget( m_useIPScan, 1, 0, 0 );
+  buttonLayout->addWidget( m_ipBroadcastAddresses, 1, 1, 0 );
 
   browsebox->insert( m_useHost, -1 );
+  browsebox->insert( m_useIPScan, -1 );
 
   //
   // The search group box
