@@ -48,7 +48,7 @@
 #include "../core/smb4kauthinfo.h"
 #include "../core/smb4kcore.h"
 #include "../core/smb4kglobal.h"
-
+#include "../core/smb4ksambaoptionsinfo.h"
 using namespace Smb4KGlobal;
 
 
@@ -69,7 +69,7 @@ Smb4KOptionsDlg::Smb4KOptionsDlg( QWidget *parent, const char *name, bool modal
 {
   setWFlags( Qt::WDestructiveClose );
 
-  setupAppearanceOptions();
+  setupUserInterfaceOptions();
   setupNetworkOptions();
   setupShareOptions();
   setupAuthOptions();
@@ -82,10 +82,10 @@ Smb4KOptionsDlg::Smb4KOptionsDlg( QWidget *parent, const char *name, bool modal
   readOptions();
 
   // Connections this the core
-  connect( smb4k_core->fileIO(),  SIGNAL( finished_suid_writing() ),
-           this,                  SLOT( slotFinishedSUIDWriting() ) );
-  connect( smb4k_core->fileIO(),  SIGNAL( error_suid_writing() ),
-           this,                  SLOT( slotSUIDWritingFailed() ) );
+  connect( smb4k_core->fileIO(),  SIGNAL( finished() ),
+           this,                  SLOT( slotFinishedWriting() ) );
+  connect( smb4k_core->fileIO(),  SIGNAL( failed() ),
+           this,                  SLOT( slotWritingFailed() ) );
 
   connect( m_superPage->m_remove, SIGNAL( clicked() ),
            this,                  SLOT( slotRemoveSuperUserEntries() ) );
@@ -176,9 +176,9 @@ void Smb4KOptionsDlg::setupAuthOptions()
    Set up the apprearance options page.
 ****************************************************************************/
 
-void Smb4KOptionsDlg::setupAppearanceOptions()
+void Smb4KOptionsDlg::setupUserInterfaceOptions()
 {
-  QFrame *frame = addPage( i18n( "Appearance" ), i18n( "Appearance" ), DesktopIcon( "colorize" ) );
+  QFrame *frame = addPage( i18n( "User Interface" ), i18n( "User Interface" ), DesktopIcon( "view_choose" ) );
 
   QGridLayout *frameLayout = new QGridLayout( frame );
   frameLayout->setSpacing( 0 );
@@ -190,8 +190,8 @@ void Smb4KOptionsDlg::setupAppearanceOptions()
   container->setFrameShape( QScrollView::MenuBarPanel );
   container->setLineWidth( 0 );
   container->setMidLineWidth( 0 );
-  m_appearancePage = new Smb4KAppearanceOptions( container->viewport() );
-  container->addChild( m_appearancePage, 0, 0 );
+  m_uiPage = new Smb4KUserInterfaceOptions( container->viewport() );
+  container->addChild( m_uiPage, 0, 0 );
 
   frameLayout->addWidget( container, 0, 0, 0 );
 }
@@ -323,8 +323,13 @@ bool Smb4KOptionsDlg::saveOptions()
   {
     config()->writeEntry( "Browse List", "host" );
   }
+  else if ( m_networkPage->m_useIPScan->isChecked() )
+  {
+    config()->writeEntry( "Browse List", "ipscan" );
+  }
 
   config()->writeEntry( "Query Host", m_networkPage->m_hostName->text().stripWhiteSpace().upper() );
+  config()->writeEntry( "IP Scan Broadcast Addresses", m_networkPage->m_ipBroadcastAddresses->text().stripWhiteSpace() );
 
   if ( m_networkPage->m_nmbSearch->isChecked() )
   {
@@ -337,30 +342,24 @@ bool Smb4KOptionsDlg::saveOptions()
 
 
   //
-  // Appearance
+  // User interface
   //
-  config()->setGroup( "Appearance" );
-  config()->writeEntry( "Show Hidden", m_appearancePage->m_showHidden->isChecked() );
-  config()->writeEntry( "Show IPC", m_appearancePage->m_showIPC->isChecked() );
-  config()->writeEntry( "Show ADMIN", m_appearancePage->m_showAdmin->isChecked() );
-  config()->writeEntry( "Show Printer", m_appearancePage->m_showPrinter->isChecked() );
-  config()->writeEntry( "Show IP", m_appearancePage->m_showIP->isChecked() );
-  config()->writeEntry( "Show Type", m_appearancePage->m_showType->isChecked() );
-  config()->writeEntry( "Show Comment", m_appearancePage->m_showComment->isChecked() );
-
-  if ( m_appearancePage->m_shareview->isChecked() )
-  {
-    config()->writeEntry( "Show Shares", "list" );
-  }
-  else
-  {
-    config()->writeEntry( "Show Shares", "icons" );
-  }
-
-  config()->writeEntry( "Show Mount Point", m_appearancePage->m_description->isChecked() );
-  config()->writeEntry( "Show All Shares", m_appearancePage->m_external->isChecked() );
-  config()->writeEntry( "Show Share Tooltip", m_appearancePage->m_showShareTooltip->isChecked() );
-  config()->writeEntry( "Show Network Tooltip", m_appearancePage->m_showNetworkItemTooltip->isChecked() );
+  config()->setGroup( "User Interface" );
+  config()->writeEntry( "Show Hidden", m_uiPage->m_showHidden->isChecked() );
+  config()->writeEntry( "Show IPC", m_uiPage->m_showIPC->isChecked() );
+  config()->writeEntry( "Show ADMIN", m_uiPage->m_showAdmin->isChecked() );
+  config()->writeEntry( "Show Printer", m_uiPage->m_showPrinter->isChecked() );
+  config()->writeEntry( "Show IP", m_uiPage->m_showIP->isChecked() );
+  config()->writeEntry( "Show Type", m_uiPage->m_showType->isChecked() );
+  config()->writeEntry( "Show Comment", m_uiPage->m_showComment->isChecked() );
+  config()->writeEntry( "Show Shares", m_uiPage->m_shareview->isChecked() ? "list" : "icons" );
+  config()->writeEntry( "Show Mount Point", m_uiPage->m_description->isChecked() );
+  config()->writeEntry( "Show All Shares", m_uiPage->m_external->isChecked() );
+  config()->writeEntry( "Show Share Tooltip", m_uiPage->m_showShareTooltip->isChecked() );
+  config()->writeEntry( "Show Network Tooltip", m_uiPage->m_showNetworkItemTooltip->isChecked() );
+  config()->writeEntry( "Allow Dropping", m_uiPage->m_dropSupport->isChecked() );
+  config()->writeEntry( "Allow Dragging", m_uiPage->m_dragSupport->isChecked() );
+  config()->writeEntry( "Preview Hidden", m_uiPage->m_previewHiddenItems->isChecked() );
 
   //
   // Authentication
@@ -373,7 +372,7 @@ bool Smb4KOptionsDlg::saveOptions()
   if ( m_authPage->m_defaultAuth->isChecked() )
   {
     Smb4KAuthInfo *auth = new Smb4KAuthInfo( QString::null, QString::null, QString::null, m_authPage->m_defaultLogin->text().stripWhiteSpace(), m_authPage->m_defaultPasswd->text().stripWhiteSpace() );
-    smb4k_core->passwordHandler()->writeDefaultAuth( auth );
+    passwordHandler()->writeDefaultAuth( auth );
   }
 
   //
@@ -383,7 +382,7 @@ bool Smb4KOptionsDlg::saveOptions()
 
   // General
   if ( !m_sambaPage->m_netbios->text().stripWhiteSpace().isEmpty() &&
-       QString::compare( *(globalSMBOptions().find( "netbios name" )), m_sambaPage->m_netbios->text().stripWhiteSpace().upper() ) != 0 )
+       QString::compare( *(optionsHandler()->globalSambaOptions().find( "netbios name" )), m_sambaPage->m_netbios->text().stripWhiteSpace().upper() ) != 0 )
   {
     config()->writeEntry( "NetBIOS Name", m_sambaPage->m_netbios->text().stripWhiteSpace().upper() );
   }
@@ -395,7 +394,7 @@ bool Smb4KOptionsDlg::saveOptions()
   if ( m_sambaPage->m_domain->isEnabled() )
   {
     if ( !m_sambaPage->m_domain->text().stripWhiteSpace().isEmpty() &&
-         QString::compare( m_sambaPage->m_domain->text().stripWhiteSpace().upper(), *(globalSMBOptions().find( "workgroup" )) ) != 0 )
+         QString::compare( m_sambaPage->m_domain->text().stripWhiteSpace().upper(), *(optionsHandler()->globalSambaOptions().find( "workgroup" )) ) != 0 )
     {
       config()->writeEntry( "Domain", m_sambaPage->m_domain->text().stripWhiteSpace().upper() );
     }
@@ -406,7 +405,7 @@ bool Smb4KOptionsDlg::saveOptions()
   }
 
   if ( !m_sambaPage->m_sockopt->text().stripWhiteSpace().isEmpty() &&
-       QString::compare( *(globalSMBOptions().find( "socket options" )), m_sambaPage->m_sockopt->text().stripWhiteSpace().upper() ) != 0 )
+       QString::compare( *(optionsHandler()->globalSambaOptions().find( "socket options" )), m_sambaPage->m_sockopt->text().stripWhiteSpace().upper() ) != 0 )
   {
     config()->writeEntry( "Socket Options", m_sambaPage->m_sockopt->text().stripWhiteSpace().upper() );
   }
@@ -416,7 +415,7 @@ bool Smb4KOptionsDlg::saveOptions()
   }
 
   if ( !m_sambaPage->m_scope->text().stripWhiteSpace().isEmpty() &&
-       QString::compare( *(globalSMBOptions().find( "netbios scope")), m_sambaPage->m_scope->text().stripWhiteSpace().upper() ) != 0 )
+       QString::compare( *(optionsHandler()->globalSambaOptions().find( "netbios scope")), m_sambaPage->m_scope->text().stripWhiteSpace().upper() ) != 0 )
   {
     config()->writeEntry( "Scope", m_sambaPage->m_scope->text().stripWhiteSpace().upper() );
   }
@@ -516,6 +515,7 @@ bool Smb4KOptionsDlg::saveOptions()
     config()->writeEntry( "Mount ReadWrite", false );
   }
 
+#ifndef __FreeBSD__
   if ( m_sambaPage->m_ttl->value() != 1000 && m_sambaPage->m_ttl->isEnabled() )
   {
     config()->writeEntry( "Mount Cache", m_sambaPage->m_ttl->value() );
@@ -545,6 +545,7 @@ bool Smb4KOptionsDlg::saveOptions()
 
   config()->writeEntry( "Mount RSize", m_sambaPage->m_rsize->isChecked() );
   config()->writeEntry( "Mount WSize", m_sambaPage->m_wsize->isChecked() );
+#endif
 
   // net
   if ( m_sambaPage->m_net_auto->isChecked() )
@@ -575,7 +576,7 @@ bool Smb4KOptionsDlg::saveOptions()
 
   // smbclient
   if ( !m_sambaPage->m_resolve->text().stripWhiteSpace().isEmpty() &&
-       QString::compare( *(globalSMBOptions().find( "name resolve order" )), m_sambaPage->m_resolve->text().stripWhiteSpace().upper() ) != 0 )
+       QString::compare( *(optionsHandler()->globalSambaOptions().find( "name resolve order" )), m_sambaPage->m_resolve->text().stripWhiteSpace().upper() ) != 0 )
   {
     config()->writeEntry( "Client Resolve Order", m_sambaPage->m_resolve->text().stripWhiteSpace().upper() );
   }
@@ -594,7 +595,7 @@ bool Smb4KOptionsDlg::saveOptions()
   }
 
   if ( !m_sambaPage->m_broadcast->text().stripWhiteSpace().isEmpty() &&
-       QString::compare( *(globalSMBOptions().find( "interfaces" )), m_sambaPage->m_broadcast->text().stripWhiteSpace().upper() ) != 0 )
+       QString::compare( *(optionsHandler()->globalSambaOptions().find( "interfaces" )), m_sambaPage->m_broadcast->text().stripWhiteSpace().upper() ) != 0 )
   {
     config()->writeEntry( "NMB Broadcast", m_sambaPage->m_broadcast->text().stripWhiteSpace().upper() );
   }
@@ -605,6 +606,89 @@ bool Smb4KOptionsDlg::saveOptions()
 
   config()->writeEntry( "NMB Port 137", m_sambaPage->m_udp->isChecked() );
 
+  // Custom options
+  if ( m_sambaPage->m_custom_options->childCount() != 0 )
+  {
+    // Set the new values:
+    QListViewItemIterator it( m_sambaPage->m_custom_options );
+
+    while ( it.current() )
+    {
+      QListViewItem *item = it.current();
+
+      Smb4KSambaOptionsInfo *info = optionsHandler()->findItem( item->text( Smb4KSambaOptions::ItemName ) );
+
+      if ( info )
+      {
+        switch( info->type() )
+        {
+          case Smb4KSambaOptionsInfo::Host:
+          {
+            info->setPort( item->text( Smb4KSambaOptions::Port ).toInt() );
+            info->setKerberos( (QString::compare( item->text( Smb4KSambaOptions::Kerberos ), i18n( "yes" ) ) == 0) );
+            info->setProtocol( QString::compare( item->text( Smb4KSambaOptions::Protocol ), "-" ) != 0 ?
+                               item->text( Smb4KSambaOptions::Protocol ).lower() : QString::null );
+            break;
+          }
+          case Smb4KSambaOptionsInfo::Share:
+          {
+            info->setPort( item->text( Smb4KSambaOptions::Port ).toInt() );
+#ifndef __FreeBSD__
+            info->setKerberos( (QString::compare( item->text( Smb4KSambaOptions::Kerberos ), i18n( "yes" ) ) == 0) );
+            info->setFilesystem( QString::compare( item->text( Smb4KSambaOptions::Filesystem ), "-" ) != 0 ?
+                                 (QString::compare( item->text( Smb4KSambaOptions::Filesystem ), i18n( "auto" ) ) == 0 ?
+                                 "auto" :
+                                 item->text( Smb4KSambaOptions::Filesystem ).lower()) :
+                                 QString::null );
+            info->setReadWrite( (QString::compare( item->text( Smb4KSambaOptions::WriteAccess ), i18n( "read-write" ) ) == 0) );
+#endif
+            info->setUID( (QString::compare( item->text( Smb4KSambaOptions::UID ), i18n( "default" ) ) != 0 &&
+                          QString::compare( item->text( Smb4KSambaOptions::UID ), "-" ) != 0) ?
+                          item->text( Smb4KSambaOptions::UID ) :
+                          QString::null );
+            info->setGID( (QString::compare( item->text( Smb4KSambaOptions::GID ), i18n( "default" ) ) != 0 &&
+                          QString::compare( item->text( Smb4KSambaOptions::GID ), "-" ) != 0) ?
+                          item->text( Smb4KSambaOptions::GID ) :
+                          QString::null );
+            break;
+          }
+          default:
+          {
+            break;
+          }
+        }
+      }
+      else
+      {
+        // We do not have this case.
+      }
+
+      ++it;
+    }
+  }
+  else
+  {
+    // Remove the items:
+
+    QValueList<Smb4KSambaOptionsInfo *> list = optionsHandler()->customOptionsList();
+
+    for ( QValueList<Smb4KSambaOptionsInfo *>::Iterator it = list.begin(); it != list.end(); ++it )
+    {
+      if ( !(*it)->remount() )
+      {
+        optionsHandler()->removeItem( (*it)->itemName(), false );
+
+        continue;
+      }
+      else
+      {
+        continue;
+      }
+    }
+  }
+
+  optionsHandler()->sync();
+
 
   //
   // The super user tab
@@ -621,7 +705,9 @@ bool Smb4KOptionsDlg::saveOptions()
     {
       config()->writeEntry( "SUID Program", "sudo" );
     }
+#ifdef __linux__
     config()->writeEntry( "Force Unmount", m_superPage->m_unmount_dead->isChecked() );
+#endif
     config()->writeEntry( "Run SUID", m_superPage->m_use_suid->isChecked() );
   }
   else
@@ -723,12 +809,16 @@ bool Smb4KOptionsDlg::saveOptions()
 void Smb4KOptionsDlg::readOptions()
 {
   int mask = getUMASK();
-  QString system = systemName();
-  QString kernel = kernelVersion();
 
+#ifdef __linux__
+  // We only need this for Linux, because we need to disable
+  // some widgets in case the respective features are not
+  // supported.
+  QString kernel = kernelVersion();
   int k_major = kernel.section( ".", 0, 0 ).toInt();
   int k_minor = kernel.section( ".", 1, 1 ).toInt();
   int k_rev = kernel.section( ".", 2, 2 ).toInt();
+#endif
 
   config()->setGroup( "Programs" );
   QString super = config()->readPathEntry( "super" );
@@ -748,6 +838,7 @@ void Smb4KOptionsDlg::readOptions()
   m_sharePage->m_lowercase->setChecked( config()->readBoolEntry( "Force Lowercase", false ) );
   m_sharePage->m_check_interval->setValue( config()->readNumEntry( "Check Interval", 2500 ) );
 
+
   //
   // Browse options
   //
@@ -755,6 +846,7 @@ void Smb4KOptionsDlg::readOptions()
   // Toggle the browse list buttons.
   QString browselist = config()->readEntry( "Browse List", "lookup" );
   QString host = config()->readEntry( "Query Host", QString::null );
+  QString ip_scan_broadcast_addresses = config()->readEntry( "IP Scan Broadcast Addresses", QString::null );
 
   if ( QString::compare( browselist.stripWhiteSpace(), "lookup" ) == 0 )
   {
@@ -768,12 +860,14 @@ void Smb4KOptionsDlg::readOptions()
   {
     m_networkPage->m_useHost->setChecked( true );
   }
-
-  if ( !host.isEmpty() )
+  else if ( QString::compare( browselist.stripWhiteSpace(), "ipscan" ) == 0 )
   {
-    m_networkPage->m_hostName->setText( host );
+    m_networkPage->m_useIPScan->setChecked( true );
   }
 
+  m_networkPage->m_hostName->setText( host );
+  m_networkPage->m_ipBroadcastAddresses->setText( ip_scan_broadcast_addresses );
+
   if ( QString::compare( config()->readEntry( "Network Search", "nmblookup" ), "nmblookup" ) == 0 )
   {
     m_networkPage->m_nmbSearch->setChecked( true );
@@ -785,45 +879,49 @@ void Smb4KOptionsDlg::readOptions()
 
 
   //
-  // Appearance
+  // User interface
   //
-  config()->setGroup( "Appearance" );
-  m_appearancePage->m_showHidden->setChecked( config()->readBoolEntry( "Show Hidden", true ) );
-  m_appearancePage->m_showIPC->setChecked( config()->readBoolEntry( "Show IPC", false ) );
-  m_appearancePage->m_showAdmin->setChecked( config()->readBoolEntry( "Show ADMIN", false ) );
-  m_appearancePage->m_showPrinter->setChecked( config()->readBoolEntry( "Show Printer", true ) );
-  m_appearancePage->m_showIP->setChecked( config()->readBoolEntry( "Show IP", true ) );
-  m_appearancePage->m_showType->setChecked( config()->readBoolEntry( "Show Type", true ) );
-  m_appearancePage->m_showComment->setChecked( config()->readBoolEntry( "Show Comment", true ) );
+  config()->setGroup( "User Interface" );
+  m_uiPage->m_showHidden->setChecked( config()->readBoolEntry( "Show Hidden", true ) );
+  m_uiPage->m_showIPC->setChecked( config()->readBoolEntry( "Show IPC", false ) );
+  m_uiPage->m_showAdmin->setChecked( config()->readBoolEntry( "Show ADMIN", false ) );
+  m_uiPage->m_showPrinter->setChecked( config()->readBoolEntry( "Show Printer", true ) );
+  m_uiPage->m_showIP->setChecked( config()->readBoolEntry( "Show IP", true ) );
+  m_uiPage->m_showType->setChecked( config()->readBoolEntry( "Show Type", true ) );
+  m_uiPage->m_showComment->setChecked( config()->readBoolEntry( "Show Comment", true ) );
 
   if ( QString::compare( config()->readEntry( "Show Shares", "icons" ), "icons" ) == 0 )
   {
-    m_appearancePage->m_shareview->setChecked( false );
+    m_uiPage->m_shareview->setChecked( false );
   }
   else if ( QString::compare( config()->readEntry( "Show Shares", "icons" ), "list" ) == 0 )
   {
-    m_appearancePage->m_shareview->setChecked( true );
+    m_uiPage->m_shareview->setChecked( true );
   }
 
   if ( !config()->readBoolEntry( "Show Mount Point", false ) )
   {
-    m_appearancePage->m_description->setChecked( false );
+    m_uiPage->m_description->setChecked( false );
   }
   else
   {
-    m_appearancePage->m_description->setChecked( true );
+    m_uiPage->m_description->setChecked( true );
   }
 
-  m_appearancePage->m_external->setChecked( config()->readBoolEntry( "Show All Shares", false ) );
-  m_appearancePage->m_showShareTooltip->setChecked( config()->readBoolEntry( "Show Share Tooltip", true ) );
-  m_appearancePage->m_showNetworkItemTooltip->setChecked( config()->readBoolEntry( "Show Network Tooltip", true ) );
+  m_uiPage->m_external->setChecked( config()->readBoolEntry( "Show All Shares", false ) );
+  m_uiPage->m_showShareTooltip->setChecked( config()->readBoolEntry( "Show Share Tooltip", true ) );
+  m_uiPage->m_showNetworkItemTooltip->setChecked( config()->readBoolEntry( "Show Network Tooltip", true ) );
+  m_uiPage->m_dropSupport->setChecked( config()->readBoolEntry( "Allow Dropping", false ) );
+  m_uiPage->m_dragSupport->setChecked( config()->readBoolEntry( "Allow Dragging", false ) );
+  m_uiPage->m_previewHiddenItems->setChecked( config()->readBoolEntry( "Preview Hidden", true ) );
 
 
   //
   // General options
   //
   config()->setGroup( "Dialogs" );
-  setInitialSize( config()->readSizeEntry( "Configuration Dialog Geometry", new QSize( 800, 600 ) ), false );
+  QSize initial_size = QSize( 800, 600 );
+  setInitialSize( config()->readSizeEntry( "Configuration Dialog Geometry", &initial_size  ), false );
 
 
   //
@@ -833,7 +931,7 @@ void Smb4KOptionsDlg::readOptions()
 
   if ( config()->readBoolEntry( "Use Wallet", true ) )
   {
-    Smb4KAuthInfo *defaultAuth = smb4k_core->passwordHandler()->readDefaultAuth();
+    Smb4KAuthInfo *defaultAuth = passwordHandler()->readDefaultAuth();
 
     if ( defaultAuth )
     {
@@ -863,33 +961,37 @@ void Smb4KOptionsDlg::readOptions()
   // Samba options
   //
   config()->setGroup( "Samba" );
-  m_sambaPage->m_netbios->setText( config()->readEntry( "NetBIOS Name", *(globalSMBOptions().find( "netbios name" )) ) );
-  m_sambaPage->m_domain->setText( config()->readEntry( "Domain", *(globalSMBOptions().find( "workgroup" )) ) );
-  m_sambaPage->m_sockopt->setText( config()->readEntry( "Socket Options", *(globalSMBOptions().find( "socket options" )) ) );
-  m_sambaPage->m_scope->setText( config()->readEntry( "NetBIOS Scope", *(globalSMBOptions().find( "netbios scope" )) ) );
-  m_sambaPage->m_port->setValue( config()->readNumEntry( "Port", 139 ) );
+  int default_port = config()->readNumEntry( "Port", 139 );
+  QString default_filesystem = config()->readEntry( "Mount Filesystem", "smbfs" );
+  QString default_protocol = config()->readEntry( "Net Protocol", "auto" );
+  bool default_kerberos = config()->readBoolEntry( "Use Kerberos", false );
+  bool default_rw = config()->readBoolEntry( "Mount ReadWrite", true );
+  QString default_uid = config()->readEntry( "Mount UID", i18n( "default" ) );
+  QString default_gid = config()->readEntry( "Mount GID", i18n( "default" ) );
 
-  m_sambaPage->m_krb->setChecked( config()->readBoolEntry( "Use Kerberos", false ) );
-  m_sambaPage->m_unicode->setChecked( config()->readBoolEntry( "Mount Unicode", false ) );
-  m_sambaPage->m_lfs->setChecked( config()->readBoolEntry( "Mount LFS", false ) );
+  m_sambaPage->m_netbios->setText( config()->readEntry( "NetBIOS Name", *(optionsHandler()->globalSambaOptions().find( "netbios name" )) ) );
+  m_sambaPage->m_domain->setText( config()->readEntry( "Domain", *(optionsHandler()->globalSambaOptions().find( "workgroup" )) ) );
+  m_sambaPage->m_sockopt->setText( config()->readEntry( "Socket Options", *(optionsHandler()->globalSambaOptions().find( "socket options" )) ) );
+  m_sambaPage->m_scope->setText( config()->readEntry( "NetBIOS Scope", *(optionsHandler()->globalSambaOptions().find( "netbios scope" )) ) );
+  m_sambaPage->m_port->setValue( default_port );
 
-  QString filesystem = config()->readEntry( "Mount Filesystem", "smbfs" );
+  m_sambaPage->m_krb->setChecked( default_kerberos );
 
-  if ( QString::compare( filesystem, "smbfs" ) == 0 || mount_cifs.isEmpty() )
+  if ( !mount_cifs.isEmpty() )
   {
-    m_sambaPage->m_filesystem->setCurrentText( "SMBFS" );
-    m_sambaPage->slotSambaFileSystem( "SMBFS" );
+    // Set the file system and disable the respective widgets:
+    m_sambaPage->m_filesystem->setCurrentText( default_filesystem.upper() );
+    m_sambaPage->slotSambaFileSystem( default_filesystem.upper() );
   }
   else
   {
-    m_sambaPage->m_filesystem->setCurrentText( "CIFS" );
-    m_sambaPage->slotSambaFileSystem( "CIFS" );
-  }
-
-  // TODO: Check this with a system that has no mount.cifs installed!
-  if ( mount_cifs.isEmpty() )
-  {
+    // If no mount.cifs is installed, remove the CIFS entry from the combo box...
     m_sambaPage->m_filesystem->removeItem( 1 );
+
+    // ... and set the file system to 'smbfs'. Also, disable the
+    // respecive widgets.
+    m_sambaPage->m_filesystem->setCurrentText( "SMBFS" );
+    m_sambaPage->slotSambaFileSystem( "SMBFS" );
   }
 
   QValueList<Smb4KUser *> list = smb4k_core->fileIO()->getUsers();
@@ -914,10 +1016,10 @@ void Smb4KOptionsDlg::readOptions()
   gid.sort();
 
   m_sambaPage->m_uid->insertStringList( uid, -1 );
-  m_sambaPage->m_uid->setCurrentText( config()->readEntry( "Mount UID", i18n( "default" ) ) );
+  m_sambaPage->m_uid->setCurrentText( default_uid );
 
   m_sambaPage->m_gid->insertStringList( gid, -1 );
-  m_sambaPage->m_gid->setCurrentText( config()->readEntry( "Mount GID", i18n( "default" ) ) );
+  m_sambaPage->m_gid->setCurrentText( default_gid );
 
 #ifdef __linux__
   // Under Linux the codepage and charset are only supported for
@@ -939,8 +1041,7 @@ void Smb4KOptionsDlg::readOptions()
   m_sambaPage->m_charset->setCurrentText( config()->readEntry( "Mount Charset", i18n( "default" ) ) );
 #endif
 
-  QString fmask = config()->readEntry( "Mount FMASK", QString( "%1" ).arg( 0777
-- mask, 0, 8 ) );
+  QString fmask = config()->readEntry( "Mount FMASK", QString( "%1" ).arg( 0777 - mask, 0, 8 ) );
   QString dmask = config()->readEntry( "Mount DMASK", QString( "%1" ).arg( 0777 - mask, 0, 8 ) );
 
   // We need to make sure that the masks have 4 digits:
@@ -963,7 +1064,7 @@ void Smb4KOptionsDlg::readOptions()
   m_sambaPage->m_fmask->setText( fmask );
   m_sambaPage->m_dmask->setText( dmask );
 
-  if ( config()->readBoolEntry( "Mount ReadWrite", true ) )
+  if ( default_rw )
   {
     m_sambaPage->m_read_mode->setCurrentText( i18n( "read-write" ) );
   }
@@ -972,6 +1073,7 @@ void Smb4KOptionsDlg::readOptions()
     m_sambaPage->m_read_mode->setCurrentText( i18n( "read-only" ) );
   }
 
+#ifndef __FreeBSD__
   if ( k_major >= 2 && k_minor >= 4 && k_rev >= 2 )
   {
     m_sambaPage->m_ttl->setValue( config()->readNumEntry( "Mount Cache", 1000 ) );
@@ -983,39 +1085,130 @@ void Smb4KOptionsDlg::readOptions()
 
   m_sambaPage->m_rsize->setChecked( config()->readBoolEntry( "Mount RSize", false ) );
   m_sambaPage->m_wsize->setChecked( config()->readBoolEntry( "Mount WSize", false ) );
+  m_sambaPage->m_unicode->setChecked( config()->readBoolEntry( "Mount Unicode", false ) );
+  m_sambaPage->m_lfs->setChecked( config()->readBoolEntry( "Mount LFS", false ) );
+#endif
 
   m_sambaPage->m_buffer->setValue( config()->readNumEntry( "Client Buffer Size", 65520 ) );
-  m_sambaPage->m_resolve->setText( config()->readEntry( "Client Resolve Order", *(globalSMBOptions().find( "name resolve order" )) ) );
+  m_sambaPage->m_resolve->setText( config()->readEntry( "Client Resolve Order", *(optionsHandler()->globalSambaOptions().find( "name resolve order" )) ) );
 
-  m_sambaPage->m_broadcast->setText( config()->readEntry( "NMB Broadcast", *(globalSMBOptions().find( "interfaces" )) ) );
+  m_sambaPage->m_broadcast->setText( config()->readEntry( "NMB Broadcast", *(optionsHandler()->globalSambaOptions().find( "interfaces" )) ) );
   m_sambaPage->m_udp->setChecked( config()->readBoolEntry( "NMB Port 137", false ) );
 
-  QString protocol = config()->readEntry( "Net Protocol", "auto" );
-
   if ( net.isEmpty() )
   {
     ((QWidget *)m_sambaPage->child( "SambaNetTab", "QWidget" ))->setEnabled( false );
   }
 
-  if ( QString::compare( protocol, "auto" ) == 0 )
+  if ( QString::compare( default_protocol, "auto" ) == 0 )
   {
     m_sambaPage->m_net_auto->setChecked( true );
   }
-  else if ( QString::compare( protocol, "ads" ) == 0 )
+  else if ( QString::compare( default_protocol, "ads" ) == 0 )
   {
     m_sambaPage->m_net_ads->setChecked( true );
   }
-  else if ( QString::compare( protocol, "rpc" ) == 0 )
+  else if ( QString::compare( default_protocol, "rpc" ) == 0 )
   {
     m_sambaPage->m_net_rpc->setChecked( true );
   }
-  else if ( QString::compare( protocol, "rap" ) == 0 )
+  else if ( QString::compare( default_protocol, "rap" ) == 0 )
   {
     m_sambaPage->m_net_rap->setChecked( true );
   }
 
   m_sambaPage->m_net_machine_account->setChecked( config()->readBoolEntry( "Net Machine Account", false ) );
 
+  // Custom options:
+  const QValueList<Smb4KSambaOptionsInfo *> &custom_options = optionsHandler()->customOptionsList();
+
+  for ( QValueList<Smb4KSambaOptionsInfo *>::ConstIterator it = custom_options.begin();
+        it != custom_options.end(); ++it )
+  {
+    if ( (*it)->port() != default_port ||
+#ifndef __FreeBSD__
+         QString::compare( (*it)->filesystem(), default_filesystem ) != 0 ||
+         (*it)->readwrite() != default_rw ||
+#endif
+         QString::compare( (*it)->protocol(), default_protocol ) != 0 ||
+         (*it)->kerberos() != default_kerberos ||
+         QString::compare( (*it)->uid(), default_uid ) != 0 ||
+         QString::compare( (*it)->gid(), default_gid ) != 0 )
+    {
+      switch ( (*it)->type() )
+      {
+        case Smb4KSambaOptionsInfo::Host:
+        {
+          KListViewItem *newItem = new KListViewItem( m_sambaPage->m_custom_options );
+          newItem->setText( Smb4KSambaOptions::ItemName, (*it)->itemName() );
+          newItem->setText( Smb4KSambaOptions::Port, (*it)->port() != -1 ?
+                            QString( "%1" ).arg( (*it)->port() ) :
+                            QString( "%1" ).arg( default_port ) );
+          newItem->setText( Smb4KSambaOptions::Protocol, !(*it)->protocol().isEmpty() ?
+                            (QString::compare( (*it)->protocol(), "auto" ) == 0 ? i18n( "auto" ) : (*it)->protocol().upper()) :
+                            (QString::compare( default_protocol, "auto" ) == 0 ? i18n( "auto" ) : default_protocol.upper()) );
+          newItem->setText( Smb4KSambaOptions::Kerberos, (*it)->kerberos() ? i18n( "yes" ) : i18n( "no" ) );
+#ifndef __FreeBSD__
+          newItem->setText( Smb4KSambaOptions::Filesystem, "-" );
+          newItem->setText( Smb4KSambaOptions::WriteAccess, "-" );
+#endif
+          newItem->setText( Smb4KSambaOptions::UID, "-" );
+          newItem->setText( Smb4KSambaOptions::GID, "-" );
+
+          break;
+        }
+        case Smb4KSambaOptionsInfo::Share:
+        {
+          KListViewItem *newItem = new KListViewItem( m_sambaPage->m_custom_options );
+          newItem->setText( Smb4KSambaOptions::ItemName, (*it)->itemName() );
+          newItem->setText( Smb4KSambaOptions::Port, (*it)->port() != -1 ?
+                            QString( "%1" ).arg( (*it)->port() ) :
+                            QString( "%1" ).arg( default_port ) );
+          newItem->setText( Smb4KSambaOptions::Protocol, "-" );
+#ifndef __FreeBSD__
+          newItem->setText( Smb4KSambaOptions::Kerberos, (*it)->kerberos() ?
+                            i18n( "yes" ) :
+                            i18n( "no" ) );
+          newItem->setText( Smb4KSambaOptions::Filesystem, !(*it)->filesystem().isEmpty() ?
+                            (*it)->filesystem().upper() :
+                            default_filesystem.upper() );
+          newItem->setText( Smb4KSambaOptions::WriteAccess, (*it)->readwrite() ?
+                            i18n( "read-write" ) :
+                            i18n( "read-only" ) );
+#else
+          // FreeBSD does not know Kerberos support for mounting.
+          newItem->setText( Smb4KSambaOptions::Kerberos, "-" );
+#endif
+          newItem->setText( Smb4KSambaOptions::UID, !(*it)->uid().isEmpty() ?
+                            (*it)->uid() :
+                            default_uid );
+          newItem->setText( Smb4KSambaOptions::GID, !(*it)->gid().isEmpty() ?
+                            (*it)->gid() :
+                            default_gid );
+
+          break;
+        }
+        default:
+        {
+          break;
+        }
+      }
+    }
+  }
+
+  m_sambaPage->m_custom_options->sort();
+
+  m_sambaPage->m_custom_port->setValue( default_port );
+  m_sambaPage->m_custom_protocol->setCurrentText( "-" );
+  m_sambaPage->m_custom_kerberos->setCurrentText( "-" );
+#ifndef __FreeBSD__
+  m_sambaPage->m_custom_filesystem->setCurrentText( "-" );
+  m_sambaPage->m_custom_rw->setCurrentText( "-" );
+#endif
+  m_sambaPage->m_custom_uid->insertStringList( uid, -1 );
+  m_sambaPage->m_custom_uid->setCurrentText( "-" );
+  m_sambaPage->m_custom_gid->insertStringList( gid, -1 );
+  m_sambaPage->m_custom_gid->setCurrentText( "-" );
 
   //
   // Super user options
@@ -1023,7 +1216,9 @@ void Smb4KOptionsDlg::readOptions()
   config()->setGroup( "Super User Privileges" );
 
   QString super_prog = super_prog_prev = config()->readEntry( "SUID Program", "super" );
+#ifdef __linux__
   bool use_force = use_force_prev = config()->readBoolEntry( "Force Unmount", false );
+#endif
   bool use_suid = use_suid_prev = config()->readBoolEntry( "Run SUID", false );
 
   if ( super.isEmpty() && sudo.isEmpty() )
@@ -1052,7 +1247,9 @@ void Smb4KOptionsDlg::readOptions()
     }
   }
 
-  if ( QString::compare( system, "Linux" ) == 0 && k_major >= 2 && ( ( k_minor == 4 && k_rev >= 11 ) || k_minor > 4 ) )
+#ifdef __linux__
+
+  if ( k_major >= 2 && ( ( k_minor == 4 && k_rev >= 11 ) || k_minor > 4 ) )
   {
     m_superPage->m_unmount_dead->setChecked( use_force );
   }
@@ -1062,6 +1259,8 @@ void Smb4KOptionsDlg::readOptions()
     m_superPage->m_unmount_dead->setEnabled( false );
   }
 
+#endif
+
   m_superPage->m_use_suid->setChecked( use_suid );
 
   //
@@ -1161,6 +1360,17 @@ bool Smb4KOptionsDlg::check()
     }
   }
 
+  if ( m_networkPage->m_useIPScan->isChecked() && m_networkPage->m_ipBroadcastAddresses->text().stripWhiteSpace().isEmpty() )
+  {
+    ok = false;
+
+    if ( KMessageBox::warningYesNo( this, i18n( "You forgot to enter the broadcast address(es) for the IP scan.\nDo you want to enter them now?" ) ) == KMessageBox::Yes )
+    {
+      this->showPage( Network );
+      m_networkPage->m_ipBroadcastAddresses->setFocus();
+    }
+  }
+
   //
   // Shares page
   //
@@ -1168,7 +1378,7 @@ bool Smb4KOptionsDlg::check()
   {
     ok = false;
 
-    if ( KMessageBox::warningYesNo( this, i18n( "You forgot to enter the path where Smb4K will mount the shares to.\nDo you want to enter it now?" ) ) == KMessageBox::Yes )
+    if ( KMessageBox::warningYesNo( this, i18n( "You forgot to enter the mount prefix.\nDo you want to enter it now?" ) ) == KMessageBox::Yes )
     {
       this->showPage( Shares );
       m_sharePage->m_mount_prefix->setFocus();
@@ -1222,7 +1432,11 @@ bool Smb4KOptionsDlg::check()
 
 void Smb4KOptionsDlg::slotOk()
 {
+#ifdef __linux__
   bool force = m_superPage->m_unmount_dead->isChecked();
+#else
+  bool force = false;
+#endif
   bool full_use = m_superPage->m_use_suid->isChecked();
 
   if ( check() )
@@ -1273,7 +1487,11 @@ void Smb4KOptionsDlg::slotOk()
 
 void Smb4KOptionsDlg::slotApply()
 {
+#ifdef __linux__
   bool force = m_superPage->m_unmount_dead->isChecked();
+#else
+  bool force = false;
+#endif
   bool full_use = m_superPage->m_use_suid->isChecked();
 
   if ( check() )
@@ -1320,7 +1538,7 @@ void Smb4KOptionsDlg::slotApply()
 }
 
 
-void Smb4KOptionsDlg::slotFinishedSUIDWriting()
+void Smb4KOptionsDlg::slotFinishedWriting()
 {
   enableButtonOK( true );
   enableButtonApply( true );
@@ -1333,13 +1551,19 @@ void Smb4KOptionsDlg::slotFinishedSUIDWriting()
   else if ( QString::compare( m_button, "Apply" ) == 0 )
   {
     super_prog_prev = m_superPage->m_super->isChecked() ? "super" : "sudo";
+#ifdef __linux__
     use_force_prev = m_superPage->m_unmount_dead->isChecked();
+#endif
     use_suid_prev = m_superPage->m_use_suid->isChecked();
+
+    // FIXME: We want to update the contents of the fmask and dmask
+    // edit lines in the Samba page!
+
   }
 }
 
 
-void Smb4KOptionsDlg::slotSUIDWritingFailed()
+void Smb4KOptionsDlg::slotWritingFailed()
 {
   config()->setGroup( "Super User Privileges" );
 
@@ -1354,8 +1578,10 @@ void Smb4KOptionsDlg::slotSUIDWritingFailed()
 
   config()->writeEntry( "SUID Program", super_prog_prev );
 
+#ifdef __linux__
   m_superPage->m_unmount_dead->setChecked( use_force_prev );
   config()->writeEntry( "Force Unmount", use_force_prev );
+#endif
 
   m_superPage->m_use_suid->setChecked( use_suid_prev );
   config()->writeEntry( "Run SUID", use_suid_prev );
@@ -1383,7 +1609,9 @@ void Smb4KOptionsDlg::slotRemoveSuperUserEntries()
 
   if ( smb4k_core->fileIO()->removeSuperUserEntries( prog ) )
   {
+#ifdef __linux__
     m_superPage->m_unmount_dead->setChecked( false );
+#endif
     m_superPage->m_use_suid->setChecked( false );
   }
 }
