@@ -38,9 +38,24 @@
 #include "../core/smb4kprintinfo.h"
 #include "../core/smb4kcore.h"
 
-Smb4KPrintDialog::Smb4KPrintDialog( const QString &workgroup, const QString &host, const QString &ip, const QString &printer, const QString &comment, QWidget *parent, const char *name )
-: KDialogBase( Plain, i18n( "Print File" ), Ok|Cancel, Ok, parent, name, false, true ), m_workgroup( workgroup ), m_host( host ), m_ip( ip ), m_printer( printer )
+
+Smb4KPrintDialog::Smb4KPrintDialog( Smb4KShareItem *item, QWidget *parent, const char *name )
+: KDialogBase( Plain, i18n( "Print File" ), Ok|Cancel, Ok, parent, name, true, true ),
+m_item( item ), m_ip( QString::null ) /* will collect it in an instant */
 {
+  setWFlags( Qt::WDestructiveClose );
+
+  if ( !m_item )
+  {
+    close();
+  }
+
+  // Get the IP address
+  if ( Smb4KHostItem *host = smb4k_core->scanner()->getHost( m_item->host(), m_item->workgroup() ) )
+  {
+    m_ip = host->ip();
+  }
+
   QFrame *frame = plainPage();
 
   QGridLayout *grid = new QGridLayout( frame );
@@ -53,26 +68,32 @@ Smb4KPrintDialog::Smb4KPrintDialog( const QString &workgroup, const QString &hos
   p_grid->setSpacing( 10 );
 
   QLabel *printerLabel = new QLabel( i18n( "Printer:" ), p_w );
-  QLabel *printerName = new QLabel( !comment.isEmpty() ? printer+"   ("+comment+")" : printer, p_w );
+  QLabel *printerName = new QLabel( !m_item->comment().isEmpty() ? m_item->name()+"   ("+m_item->comment()+")" : m_item->name(), p_w );
   p_grid->addWidget( printerLabel, 0, 0, 0 );
   p_grid->addWidget( printerName, 0, 1, 0 );
 
   QLabel *locLabel = new QLabel( i18n( "Location:" ), p_w );
-  QLabel *loc = new QLabel( "//"+host+"/"+printer, p_w );
+  QLabel *loc = new QLabel( "//"+m_item->host()+"/"+m_item->name(), p_w );
   p_grid->addWidget( locLabel, 1, 0, 0 );
   p_grid->addWidget( loc, 1, 1, 0 );
 
-  QLabel *ipLabel = new QLabel( i18n( "IP Address:" ), p_w );
+  QLabel *ipLabel = new QLabel( i18n( "IP address:" ), p_w );
   QLabel *ipAddress = new QLabel( p_w );
-  if ( ip.stripWhiteSpace().isEmpty() )
+
+  if ( m_ip.stripWhiteSpace().isEmpty() )
+  {
     ipAddress->setText( i18n( "Unknown" ) );
+  }
   else
-    ipAddress->setText( ip );
+  {
+    ipAddress->setText( m_ip );
+  }
+
   p_grid->addWidget( ipLabel, 2, 0, 0 );
   p_grid->addWidget( ipAddress, 2, 1, 0 );
 
   QLabel *wgLabel = new QLabel( i18n( "Workgroup:" ), p_w );
-  QLabel *wg = new QLabel( workgroup, p_w );
+  QLabel *wg = new QLabel( m_item->workgroup(), p_w );
   p_grid->addWidget( wgLabel, 3, 0, 0 );
   p_grid->addWidget( wg, 3, 1, 0 );
 
@@ -109,6 +130,7 @@ Smb4KPrintDialog::Smb4KPrintDialog( const QString &workgroup, const QString &hos
 
 Smb4KPrintDialog::~ Smb4KPrintDialog()
 {
+  // Do not delete the pointer to the Smb4KShareObject here!
 }
 
 
@@ -120,12 +142,10 @@ void Smb4KPrintDialog::slotOk()
 {
   if ( !m_input->url().stripWhiteSpace().isEmpty() )
   {
-    if ( smb4k_core->print()->init( new Smb4KPrintInfo( m_workgroup, m_host, m_ip, m_printer, m_input->url().stripWhiteSpace(), m_copies->value() ) ) )
+    if ( smb4k_core->print()->print( new Smb4KPrintInfo( m_item, m_ip, m_input->url().stripWhiteSpace(), m_copies->value() ) ) )
     {
       accept();
     }
-
-//     delete info;
   }
   else
   {
