@@ -38,6 +38,9 @@
 #include <kurlrequester.h>
 #include <knuminput.h>
 
+// application includes
+#include "../core/smb4knetworkitems.h"
+
 
 /**
  * This class provides the print dialog.
@@ -49,9 +52,15 @@ class Smb4KPrintDialog : public KDialogBase
 
   public:
     /**
-     * The constructor
+     * The constructor.
+     *
+     * @param item        The Smb4KShareItem of the printer.
+     *
+     * @param parent      The parent widget of this dialog.
+     *
+     * @param name        The name of this dialog.
      */
-    Smb4KPrintDialog( const QString &workgroup, const QString &host, const QString &ip, const QString &printer, const QString &comment, QWidget *parent = 0, const char *name = 0 );
+    Smb4KPrintDialog( Smb4KShareItem *item, QWidget *parent = 0, const char *name = 0 );
     /**
      * The destructor
      */
@@ -65,21 +74,13 @@ class Smb4KPrintDialog : public KDialogBase
 
   private:
     /**
-     * Holds the workgroup.
+     * The Smb4KShareItem object.
      */
-    QString m_workgroup;
-    /**
-     * Holds the host.
-     */
-    QString m_host;
+    Smb4KShareItem *m_item;
     /**
      * Holds the IP address.
      */
     QString m_ip;
-    /**
-     * Holds the printer name.
-     */
-    QString m_printer;
     /**
      * The URL requester object, that is needed to specify the
      * the file to print.
