@@ -45,37 +45,34 @@ Smb4KShareOptions::Smb4KShareOptions( QWidget *parent, const char *name ) : QWid
   QGridLayout *grid = new QGridLayout( this );
   grid->setSpacing( 10 );
 
-  QVGroupBox *dirBox = new QVGroupBox( i18n( "Directories" ), this );
+  QVGroupBox *dir_box = new QVGroupBox( i18n( "Directories" ), this );
+  dir_box->setInsideSpacing( 10 );
 
-  QWidget *dir_container = new QWidget( dirBox, "DirectoryContainer" );
-  QGridLayout *dir_layout = new QGridLayout( dir_container );
-  dir_layout->setSpacing( 10 );
-
-  QLabel *mount_prefix_label = new QLabel( i18n( "Mount prefix:" ), dir_container );
-  dir_layout->addWidget( mount_prefix_label, 0, 0, 0 );
+  QGroupBox *dir_container = new QGroupBox( 2, QGroupBox::Horizontal, dir_box );
+  dir_container->setInsideMargin( 0 );
+  dir_container->setInsideSpacing( 10 );
+  dir_container->setLineWidth( 0 );
+  (void) new QLabel( i18n( "Mount prefix:" ), dir_container );
   m_mount_prefix = new KURLRequester( QString::null, dir_container );
   m_mount_prefix->setMode( KFile::Directory );
-  dir_layout->addWidget( m_mount_prefix, 0, 1, 0 );
+  m_lowercase = new QCheckBox( i18n( "Force generated subdirectories to be lower case" ), dir_box );
 
-  m_lowercase = new QCheckBox( i18n( "Force generated subdirectories to be lower case" ), dir_container );
-  dir_layout->addMultiCellWidget( m_lowercase, 1, 1, 0, 1, 0 );
+  QButtonGroup *mount_box = new QButtonGroup( 1, QButtonGroup::Horizontal, i18n( "Mounting and Unmounting" ), this );
+  m_unmountAll = new QCheckBox( i18n( "Unmount all shares of user %1 on exit" ).arg( getenv( "USER" ) ), mount_box );
+  m_mountRecent = new QCheckBox( i18n( "Remount recently used shares on program start" ), mount_box );
+  m_allowUnmountForeign = new QCheckBox( i18n( "Allow the unmounting of shares that are owned by other users" ), mount_box );
 
-  QButtonGroup *mountBox = new QButtonGroup( 1, QButtonGroup::Horizontal, i18n( "Mounting and Unmounting" ), this );
-  m_unmountAll = new QCheckBox( i18n( "Unmount all shares of user %1 on exit" ).arg( getenv( "USER" ) ), mountBox );
-  m_mountRecent = new QCheckBox( i18n( "Remount recently used shares on program start" ), mountBox );
-  m_allowUnmountForeign = new QCheckBox( i18n( "Allow the unmounting of shares that are owned by other users" ), mountBox );
-
-  QGroupBox *checks = new QGroupBox( 2, QGroupBox::Horizontal, i18n( "Checks" ), this );
-  (void) new QLabel( i18n( "Interval between checks:" ), checks );
-  m_check_interval = new KIntNumInput( 2500, checks, 10, "IntervalInput" );
+  QGroupBox *checks_box = new QGroupBox( 2, QGroupBox::Horizontal, i18n( "Checks" ), this );
+  (void) new QLabel( i18n( "Interval between checks:" ), checks_box );
+  m_check_interval = new KIntNumInput( 2500, checks_box, 10, "IntervalInput" );
   m_check_interval->setSuffix( " ms" );
   m_check_interval->setRange( 500, 300000, 1, false );
 
   QSpacerItem *spacer2 = new QSpacerItem( 0, 0, QSizePolicy::Preferred, QSizePolicy::Expanding );
 
-  grid->addWidget( dirBox, 0, 0, 0 );
-  grid->addWidget( mountBox, 1, 0, 0 );
-  grid->addWidget( checks, 2, 0, 0 );
+  grid->addWidget( dir_box, 0, 0, 0 );
+  grid->addWidget( mount_box, 1, 0, 0 );
+  grid->addWidget( checks_box, 2, 0, 0 );
   grid->addItem( spacer2, 3, 0 );
 }
 
