@@ -34,7 +34,7 @@
 #include <cerrno>
 using namespace std;
 
-#define SMB4K_CAT_VERSION 0.2
+#define SMB4K_CAT_VERSION 0.3
 
 //
 // Usage
@@ -54,36 +54,6 @@ void info()
   cout << endl;
 }
 
-//
-// Find the config file
-//
-
-const char *findfile( const char *name )
-{
-  const char *paths[] = { "./", "/etc/", "/usr/local/etc", "/opt/etc/" };
-  string file = "";
-
-  for ( uint i = 0; i < sizeof( paths ) / sizeof( char * ); i++ )
-  {
-    string path( paths[i] );
-    path.append( name );
-
-    if ( access( path.c_str(), F_OK ) == 0 )
-    {
-      file.assign( path );
-      break;
-    }
-  }
-
-  if ( !strcmp( file.c_str(), "" ) )
-  {
-    cerr << "smb4k_cat: Could not find " << name << "." << endl;
-    exit( EXIT_FAILURE );
-  }
-
-  return file.c_str();
-}
-
 //
 // The main program
 //
@@ -106,14 +76,8 @@ int main( int argc, char *argv[] )
     {
       char config[255];
 
-      if ( !strchr( argv[1], '/' ) )
-      {
-        strcpy( config, findfile( argv[1] ) );
-      }
-      else
-      {
-        strcpy( config, argv[1] );
-      }
+      (void) strncpy( config, argv[1], 254 );
+      config[254] = '\0';
 
       if ( strlen( config ) != 0 )
       {
