@@ -34,7 +34,7 @@
 #include <unistd.h>
 using namespace std;
 
-#define SMB4K_KILL_VERSION 0.3
+#define SMB4K_KILL_VERSION 0.5
 
 
 //
@@ -105,7 +105,11 @@ int main( int argc, char *argv[], char *envp[] )
 
   char *p = new char[100];
 
-  args[k] = strcpy( p, findprog( "kill" ) );
+  const char *prg = findprog( "kill" );
+
+  args[k] = strncpy( p, prg, 99 );
+  p[99] = '\0';    // Do this for security reasons (see #9631)
+
   k++;
 
   for ( int i = 1; i < argc; i++ )
@@ -127,11 +131,12 @@ int main( int argc, char *argv[], char *envp[] )
 
       if ( strlen( argv[i] ) > 100 )
       {
-        cerr << "smb4k_kill: One or more arguments exceed 100 characters." << endl;
+        cerr << "smb4k_kill: One or more arguments exceed 100 characters" << endl;
         exit( EXIT_FAILURE );
       }
 
-      args[k] = strcpy( t, argv[i] );
+      args[k] = strncpy( t, argv[i], 99 );
+      t[99] = '\0';
       k++;
 
       continue;
@@ -140,6 +145,12 @@ int main( int argc, char *argv[], char *envp[] )
 
   args[k] = NULL;
 
+  if ( k >= 25 )
+  {
+    cerr << "smb4k_kill: There are too many arguments" << endl;
+    exit( EXIT_FAILURE );
+  }
+
   if ( execve( args[0], args, envp ) == -1 )
   {
     int err_code = errno;
