@@ -34,7 +34,7 @@
 #include <unistd.h>
 using namespace std;
 
-#define SMB4K_MOUNT_VERSION 0.5
+#define SMB4K_MOUNT_VERSION 0.7
 
 //
 // Returns the information about the program
@@ -297,7 +297,9 @@ int main( int argc, char *argv[], char *envp[] )
         exit( EXIT_FAILURE );
       }
 
-      args[k] = strcpy( p, findprog( "mount_smbfs" ) );
+      const char *mnt_prg = findprog( "mount_smbfs" );
+      args[k] = strncpy( p, mnt_prg, 99 );
+      p[99] = '\0';
 #endif
 
       k++;
@@ -320,7 +322,9 @@ int main( int argc, char *argv[], char *envp[] )
         exit( EXIT_FAILURE );
       }
 
-      args[k] = strcpy( p, findprog( "mount_smbfs" ) );
+      const char *mnt_prg = findprog( "mount_smbfs" );
+      args[k] = strncpy( p, mnt_prg, 99 );
+      p[99] = '\0';
 #endif
 
       k++;
@@ -356,7 +360,9 @@ int main( int argc, char *argv[], char *envp[] )
                 exit( EXIT_FAILURE );
               }
 
-              args[0] = strcpy( p, findprog( "mount" ) );
+              const char *mnt_prg = findprog( "mount" );
+              args[0] = strncpy( p, mnt_prg, 99 );
+              p[99] = '\0';
             }
             else
             {
@@ -370,11 +376,15 @@ int main( int argc, char *argv[], char *envp[] )
 
               if ( !strcmp( argv[i+1], "smbfs" ) )
               {
-                args[0] = strcpy( p, findprog( "smbmount" ) );
+                const char *mnt_prg = findprog( "smbmount" );
+                args[0] = strncpy( p, mnt_prg, 99 );
+                p[99] = '\0';
               }
               else if ( !strcmp( argv[i+1], "cifs" ) )
               {
-                args[0] = strcpy( p, findprog( "mount.cifs" ) );
+                const char *mnt_prg = findprog( "mount.cifs" );
+                args[0] = strncpy( p, mnt_prg, 99 );
+                p[99] = '\0';
               }
 
               // For smbmount and mount.cifs, we do not need the -t option.
@@ -393,7 +403,7 @@ int main( int argc, char *argv[], char *envp[] )
       }
 #endif
 
-      char *t = new char[strlen( argv[i] )+1];
+      char *t = new char[500];
 
       if ( !t )
       {
@@ -401,7 +411,9 @@ int main( int argc, char *argv[], char *envp[] )
         exit( EXIT_FAILURE );
       }
 
-      args[k] = strcpy( t, replace_special_characters( argv[i] ) );
+      const char *rsc = replace_special_characters( argv[i] );
+      args[k] = strncpy( t, rsc, 499 );
+      t[499] = '\0';
       k++;
 
       continue;
@@ -410,6 +422,12 @@ int main( int argc, char *argv[], char *envp[] )
 
   args[k] = NULL;
 
+  if ( k >= 25 )
+  {
+    cerr << "smb4k_mount: There are too many arguments" << endl;
+    exit( EXIT_FAILURE );
+  }
+
   // FIXME: Do we need the following under FreeBSD?
 
   if ( !suid_arg_exists )
