@@ -41,12 +41,10 @@
 #elif defined(__FreeBSD__)
 #include <sys/param.h>
 #include <sys/mount.h>
-#elif defined(__irix__)
-#include <sys/types.h>
 #endif
 using namespace std;
 
-#define SMB4K_UMOUNT_VERSION 0.9
+#define SMB4K_UMOUNT_VERSION 0.11
 
 //
 // Returns the information about the program
@@ -304,23 +302,27 @@ bool check( const char *path, char *fs_buf )
 
   bool ok = false;
 
-#if !defined(__FreeBSD__) && !defined(__solaris__) && !defined(USE_SOLARIS) && !defined(__irix__)
+#if !defined(__FreeBSD__) && !defined(__solaris__) && !defined(USE_SOLARIS)
   // First entry is for CIFS, the second for SMBFS.
   if ( (uint)filesystem.f_type == 0xFF534D42 || (uint)filesystem.f_type == 0x517B )
   {
-    strcpy( fs_buf, (uint)filesystem.f_type == 0xFF534D42 ? "cifs" : "smbfs" );
+    const char *fs_type = (uint)filesystem.f_type == 0xFF534D42 ? "cifs" : "smbfs";
+    (void) strncpy( fs_buf, fs_type, strlen( fs_type ) + 1 );
 #elif defined(__FreeBSD__)
   if ( !strcmp( filesystem.f_fstypename, "smbfs" ) )
   {
-    strcpy( fs_buf, "smbfs" );
+    const char *fs_type = "smbfs";
+    (void) strncpy( fs_buf, fs_type, strlen( fs_type ) + 1 );
 #elif defined(__solaris__) || defined(USE_SOLARIS)
   if ( (uint)filesystem.f_basetype == 0xFF534D42 || (uint)filesystem.f_basetype == 0x517B )
   {
-    strcpy( fs_buf, (uint)filesystem.f_basetype == 0xFF534D42 ? "cifs" : "smbfs" );
+    const char *fs_type = (uint)filesystem.f_basetype == 0xFF534D42 ? "cifs" : "smbfs";
+    (void) strncpy( fs_buf, fs_type, strlen( fs_type ) + 1 );
 #elif defined(__irix__)
   if ( (uint)filesystem.f_fstyp == 0xFF534D42 || (uint)filesystem.f_fstyp == 0x517B )
   {
-    strcpy( fs_buf, (uint)filesystem.f_fstyp == 0xFF534D42 ? "cifs" : "smbfs" );
+    const char *fs_type = (uint)filesystem.f_fstyp == 0xFF534D42 ? "cifs" : "smbfs"
+    (void) strncpy( fs_buf, fs_type, strlen( fs_type ) + 1 );
 #endif
     ok = true;
   }
@@ -375,7 +377,8 @@ int main( int argc, char *argv[], char *envp[] )
     }
     else if ( !strcmp( argv[i], "--cifs" ) || !strcmp( argv[i], "--smbfs" ) )
     {
-      (void) strcpy( fs_arg, argv[i] );
+      (void) strncpy( fs_arg, argv[i], 9 );
+      fs_arg[9] = '\0';
 
       continue;
     }
@@ -387,7 +390,7 @@ int main( int argc, char *argv[], char *envp[] )
 
       if ( argv[i][0] == '/' || ((argv[i][0] == '\047' /* ' */|| argv[i][0] == '\042' /* " */) && argv[i][1] == '/') )
       {
-        char *path = new char[strlen( argv[i] )+1];
+        char *path = new char[500];
 
         if ( !path )
         {
@@ -395,7 +398,9 @@ int main( int argc, char *argv[], char *envp[] )
           exit( EXIT_FAILURE );
         }
 
-        args[k++] = strcpy( path, replace_special_characters( argv[i] ) );
+        const char *rsc = replace_special_characters( argv[i] );
+        args[k++] = strncpy( path, rsc, 499 );
+        path[499] = '\0';
 
         char fs[6];
 
@@ -415,11 +420,15 @@ int main( int argc, char *argv[], char *envp[] )
             {
               if ( !strcmp( fs, "smbfs" ) )
               {
-                args[0] = strcpy( t, findprog( "smbumount" ) );
+                const char *umnt_prg = findprog( "smbumount" );
+                args[0] = strncpy( t, umnt_prg, 254 );
+                t[254] = '\0';
               }
               else if ( !strcmp( fs, "cifs" ) )
               {
-                args[0] = strcpy( t, findprog( "umount.cifs" ) );
+                const char *umnt_prg = findprog( "umount.cifs" );
+                args[0] = strncpy( t, umnt_prg, 254 );
+                t[254] = '\0';
               }
               else
               {
@@ -427,11 +436,15 @@ int main( int argc, char *argv[], char *envp[] )
                 // file system info.
                 if ( !strcmp( fs_arg, "--smbfs" ) )
                 {
-                  args[0] = strcpy( t, findprog( "smbumount" ) );
+                  const char *umnt_prg = findprog( "smbumount" );
+                  args[0] = strncpy( t, umnt_prg, 254 );
+                  t[254] = '\0';
                 }
                 else if ( !strcmp( fs_arg, "--cifs" ) )
                 {
-                  args[0] = strcpy( t, findprog( "umount.cifs" ) );
+                  const char *umnt_prg = findprog( "umount.cifs" );
+                  args[0] = strncpy( t, umnt_prg, 254 );
+                  t[254] = '\0';
                 }
                 else
                 {
@@ -447,13 +460,17 @@ int main( int argc, char *argv[], char *envp[] )
             {
               if ( !strcmp( fs, "smbfs" ) || !strcmp( fs, "cifs" ) )
               {
-                args[0] = strcpy( t, findprog( "umount" ) );
+                const char *umnt_prg = findprog( "umount" );
+                args[0] = strncpy( t, umnt_prg, 254 );
+                t[254] = '\0';
               }
               else
               {
                 if ( !strcmp( fs_arg, "--smbfs" ) || !strcmp( fs_arg, "--cifs" ) )
                 {
-                  args[0] = strcpy( t, findprog( "umount" ) );
+                  const char *umnt_prg = findprog( "umount" );
+                  args[0] = strncpy( t, umnt_prg, 254 );
+                  t[254] = '\0';
                 }
                 else
                 {
@@ -482,7 +499,7 @@ int main( int argc, char *argv[], char *envp[] )
       }
       else
       {
-        char *t = new char[strlen( argv[i] )+1];
+        char *t = new char[500];
 
         if ( !t )
         {
@@ -490,7 +507,8 @@ int main( int argc, char *argv[], char *envp[] )
           exit( EXIT_FAILURE );
         }
 
-        args[k++] = strcpy( t, argv[i] );
+        args[k++] = strncpy( t, argv[i], 499 );
+        t[499] = '\0';
 
         continue;
       }
@@ -499,6 +517,12 @@ int main( int argc, char *argv[], char *envp[] )
 
   args[k] = NULL;
 
+  if ( k >= 5 )
+  {
+    cerr << "smb4k_umount: Too many arguments" << endl;
+    exit( EXIT_FAILURE );
+  }
+
   if ( execve( args[0], args, envp ) == -1 )
   {
     int err_code = errno;
